#Copyright (C) Arto Jantunen <viiru@iki.fi>

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import unittest
from datetime import date
from mock import Mock

import pytrainer.lib.localization
from pytrainer.lib.uc import *

pytrainer.lib.localization.initialize_gettext("../../locale")

class UCUtilTest(unittest.TestCase):

    def test_uc_pace2float(self):
        self.assertEquals(4.1, pace2float('4:06'))

    def test_uc_float2pace(self):
        self.assertEquals('4:06', float2pace(4.1))

class UCTest(unittest.TestCase):

    def setUp(self):
        self.uc = UC()
        self.uc.set_us(False)

    def tearDown(self):
        self.uc = None

    def test_uc_units(self):
        self.assertEquals(self.uc.unit_distance, "km")
        self.assertEquals(self.uc.unit_speed, "km/h")
        self.assertEquals(self.uc.unit_pace, "min/km")
        self.assertEquals(self.uc.unit_height, "m")
        self.assertEquals(self.uc.unit_weight, "kg")
        self.uc.set_us(True)
        self.assertEquals(self.uc.unit_distance, "mi")
        self.assertEquals(self.uc.unit_speed, "mph")
        self.assertEquals(self.uc.unit_pace, "min/mi")
        self.assertEquals(self.uc.unit_height, "ft")
        self.assertEquals(self.uc.unit_weight, "lb")

    def test_uc_conversions(self):
        self.assertEquals(self.uc.distance(10), 10)
        self.assertEquals(self.uc.speed(10), 10)
        self.assertEquals(self.uc.pace(10), 10)
        self.assertEquals(self.uc.height(10), 10)
        self.assertEquals(self.uc.weight(10), 10)
        self.uc.set_us(True)
        self.assertEquals(self.uc.distance(10), 6.21371192)
        self.assertEquals(self.uc.speed(10), 6.21371192)
        self.assertEquals(self.uc.pace(10), 16.09344)
        self.assertEquals(self.uc.height(10), 32.808399)
        self.assertEquals(self.uc.weight(10), 22.046239999999997)



# -*- coding: iso-8859-1 -*-

#Copyright (C) Nathan Jones ncjones@users.sourceforge.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import unittest
import pytrainer.util.date
from pytrainer.util.date import DateRange
from datetime import date
from datetime import datetime
from mock import Mock

class DateModuleTest(unittest.TestCase):
    
    def test_first_day_of_week_should_be_non_negative_integer(self):
        first_day = pytrainer.util.date.first_day_of_week()
        self.assertTrue(first_day >= 0)
    
    def test_first_day_of_week_should_be_less_than_seven(self):
        first_day = pytrainer.util.date.first_day_of_week()
        self.assertTrue(first_day < 7)

class DateRangeTest(unittest.TestCase):
    
    def setUp(self):
        pytrainer.util.date.first_day_of_week = Mock()
        pytrainer.util.date.first_day_of_week.return_value = 0

    def test_constructor_should_accept_end_date_same_as_start_date(self):
        DateRange(date(2011, 9, 28), date(2011, 9, 28))

    def test_constructor_should_reject_none_start_date(self):
        try:
            DateRange(None, date(2011, 9, 28))
        except(TypeError):
            pass
        else:
            self.fail()

    def test_constructor_should_reject_none_end_date(self):
        try:
            DateRange(date(2011, 9, 28), None)
        except(TypeError):
            pass
        else:
            self.fail()

    def test_constructor_should_reject_end_date_before_start_date(self):
        date_before = date(2011, 8, 31)
        date_after = date(2011, 9, 28)
        try:
            DateRange(date_after, date_before)
        except(ValueError):
            pass
        else:
            self.fail()
    
    def test_constructor_should_reject_date_rime_start_date(self):
        try:
            DateRange(datetime(2011, 9, 28), date(2011, 9, 29))
        except(TypeError):
            pass
        else:
            self.fail()
        pass
    
    def test_constructor_should_reject_date_rime_end_date(self):
        try:
            DateRange(date(2011, 9, 28), datetime(2011, 9, 29))
        except(TypeError):
            pass
        else:
            self.fail()
        pass
    
    def test_start_date_should_return_constructed_Value(self):
        date_range = DateRange(date(2011, 9, 28), date(2011, 9, 29))
        self.assertEquals(date(2011, 9, 28), date_range.start_date)
    
    def test_end_date_should_return_constructed_Value(self):
        date_range = DateRange(date(2011, 9, 28), date(2011, 9, 29))
        self.assertEquals(date(2011, 9, 29), date_range.end_date)
    
    def test_start_date_should_be_immutable(self):
        date_range = DateRange(date(2011, 9, 28), date(2011, 9, 29))
        try:
            date_range.start_date = date(2011, 9, 27)
        except(AttributeError):
            pass
        else:
            self.fail()
    
    def test_end_date_should_be_immutable(self):
        date_range = DateRange(date(2011, 9, 28), date(2011, 9, 29))
        try:
            date_range.end_date = date(2011, 9, 30)
        except(AttributeError):
            pass
        else:
            self.fail()
            
    def test_date_range_for_year_should_start_jan_1(self):
        date_range = DateRange.for_year_containing(date(2011, 12, 11))
        self.assertEquals(date(2011, 1, 1), date_range.start_date)
    
    def test_date_range_for_year_should_end_dec_31(self):
        date_range = DateRange.for_year_containing(date(2011, 12, 11))
        self.assertEquals(date(2011, 12, 31), date_range.end_date)
        
    def test_date_range_for_month_should_start_first_day_of_month(self):
        date_range = DateRange.for_month_containing(date(2011, 12, 11))
        self.assertEquals(date(2011, 12, 1), date_range.start_date)
        
    def test_date_range_for_month_dec_should_end_dec_31(self):
        date_range = DateRange.for_month_containing(date(2011, 12, 11))
        self.assertEquals(date(2011, 12, 31), date_range.end_date)
        
    def test_date_range_for_month_feb_should_end_feb_28(self):
        date_range = DateRange.for_month_containing(date(2011, 2, 11))
        self.assertEquals(date(2011, 2, 28), date_range.end_date)
        
    def test_date_range_for_week_should_start_sunday(self):
        date_range = DateRange.for_week_containing(date(2011, 12, 7))
        self.assertEquals(date(2011, 12, 4), date_range.start_date)
    
    def test_date_range_for_week_should_end_saturday(self):
        date_range = DateRange.for_week_containing(date(2011, 12, 7))
        self.assertEquals(date(2011, 12, 10), date_range.end_date)
        



# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net
# Modified by dgranda

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import logging
import os
from lxml import etree
from pytrainer.lib.xmlUtils import XMLParser
from pytrainer.gui.dialogs import fileChooserDialog, guiFlush

class garminTCXv2():
	def __init__(self, parent = None, validate=False):
		self.parent = parent
		self.pytrainer_main = parent.pytrainer_main
		self.tmpdir = self.pytrainer_main.profile.tmpdir
		self.data_path = os.path.dirname(__file__)
		self.validate = validate
		self.sport = self.getConfValue("Force_sport_to")

	def getConfValue(self, confVar):
		info = XMLParser(self.data_path+"/conf.xml")
		code = info.getValue("pytrainer-plugin","plugincode")
		plugindir = self.pytrainer_main.profile.plugindir
		if not os.path.isfile(plugindir+"/"+code+"/conf.xml"):
			value = None
		else:
			info = XMLParser(plugindir+"/"+code+"/conf.xml")
			value = info.getValue("pytrainer-plugin",confVar)
		return value

	def run(self):
		logging.debug(">>")
		# able to select multiple files....
		selectedFiles = fileChooserDialog(title="Choose a TCX file (or files) to import", multiple=True).getFiles()
		guiFlush()
		importfiles = []
		if not selectedFiles: #Nothing selected
			return importfiles
		for filename in selectedFiles: #Multiple files
			if self.valid_input_file(filename): #TODO could consolidate tree generation here
				tree = etree.ElementTree(file=filename)
				#Possibly multiple entries in file
				activities = self.getActivities(tree)
				for activity in activities:
					if not self.inDatabase(activity):
						sport = self.getSport(activity)
						gpxfile = "%s/garmin-tcxv2-%d.gpx" % (self.tmpdir, len(importfiles))
						self.createGPXfile(gpxfile, activity)
						importfiles.append((gpxfile, sport))
					else:
						logging.debug("File:%s activity %d already in database. Skipping import." % (filename, activities.index(activity)) )
			else:
				logging.info("File %s failed validation" % (filename))
		logging.debug("<<")
		return importfiles

	def valid_input_file(self, filename):
		""" Function to validate input file if requested"""
		if not self.validate:  #not asked to validate
			logging.debug("Not validating %s" % (filename) )
			return True
		else:
			xslfile = os.path.realpath(self.pytrainer_main.data_path)+ "/schemas/GarminTrainingCenterDatabase_v2.xsd"
			from pytrainer.lib.xmlValidation import xmlValidator
			validator = xmlValidator()
			return validator.validateXSL(filename, xslfile)

	def getActivities(self, tree):
		'''Function to return all activities in Garmin training center version 2 file
		'''
		root = tree.getroot()
		activities = root.findall(".//{http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v2}Activity")
		return activities

	def inDatabase(self, activity):
		#comparing date and start time (sport may have been changed in DB after import)
		time = self.detailsFromTCX(activity)
		if self.pytrainer_main.ddbb.select("records","*","date_time_utc=\"%s\"" % (time)):
			return True
		else:
			return False

	def getSport(self, activity):
		#return sport from file or overide if present
		if self.sport:
			return self.sport
		#sportElement = activity.find(".//{http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v2}Activity")
		try:
			sport = activity.get("Sport")
		except:
			sport = "import"
		return sport

	def detailsFromTCX(self, activity):
		timeElement = activity.find(".//{http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v2}Id")
		if timeElement is None:
			return None
		else:
			return timeElement.text

	def createGPXfile(self, gpxfile, activity):
		""" Function to transform a Garmin Training Center v2 Track to a valid GPX+ file
		"""
		xslt_doc = etree.parse(self.data_path+"/translate.xsl")
		transform = etree.XSLT(xslt_doc)
		#xml_doc = etree.parse(filename)
		xml_doc = activity
		result_tree = transform(xml_doc)
		result_tree.write(gpxfile, xml_declaration=True, encoding='UTF-8')



# -*- coding: iso-8859-1 -*-

#Copyright (C) Nathan Jones ncjones@users.sourceforge.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import logging
from lxml import etree
import os
import pytrainer
from pytrainer.upgrade.context import UpgradeContext
from pytrainer.upgrade.migratedb import MigratableDb

def initialize_data(ddbb, conf_dir):
    """Initializes the installation's data."""
    db_url = ddbb.get_connection_url()
    migratable_db = MigratableDb(os.path.dirname(__file__), db_url)
    InstalledData(migratable_db, ddbb, LegacyVersionProvider(conf_dir), UpgradeContext(conf_dir, db_url)).update_to_current()
        
class InstalledData(object):
    
    """Encapsulates an installation's existing data and provides means to
    check version state and upgrade."""
    
    def __init__(self, migratable_db, ddbb, legacy_version_provider, upgrade_context):
        self._migratable_db = migratable_db
        self._ddbb = ddbb
        self._legacy_version_provider = legacy_version_provider
        self._upgrade_context = upgrade_context
        
    def update_to_current(self):
        """Check the current state of the installation's data and update them
        if necessary so they are compatible with the latest version.
    
        The update steps depend on the state of the installation's data. The
        possible states and the update steps from those states are:
    
        1. Current (data is up to date):
            - do nothing
        2. Fresh (new installation):
            - initialise empty db
            - initialise db version metadata
        3. Stale: (data requires upgrading):
            - run upgrade scripts
        4. Legacy (data requires upgrading but is missing version metadata):
            - initialise db version metadata
            - run upgrade scripts
        
        """
        data_state = self.get_state()
        logging.info("Initializing data. Data state is: '%s'.", data_state)
        data_state.update_to_current(self)
        
    def get_state(self):
        """Get the current state of the installation's data.
        
        raises DataInitializationException if the existing data is not
        compatible and cannot be upgraded.
        """
        version = self.get_version()
        available_version= self.get_available_version()
        if self.is_versioned():
            if version == available_version:
                return DataState.CURRENT
            elif version > available_version:
                raise DataInitializationException("Existing data version ({0}) is greater than available version ({1}).".format(version, available_version))
            else:
                return DataState.STALE
        else:
            if version == None:
                if self.is_fresh():
                    return DataState.FRESH
                else:
                    raise DataInitializationException("Existing data version cannot be determined.")
            else:
                return DataState.LEGACY
            
    def is_fresh(self):
        """Check if this is a fresh installation."""
        return self._migratable_db.is_empty()
    
    def get_version(self):
        """Get the version number of an installation's data.
        
        If the data version cannot be determined then None is returned."""
        if self.is_versioned():
            return self._migratable_db.get_version()
        else:
            # Calculate data version in older version that does not use the
            # current data versioning scheme.
            legacy_version = self._legacy_version_provider.get_legacy_version()
            if legacy_version is not None:
                legacy_version = int(legacy_version)
                if legacy_version == 1: # 1.7.1
                    return 1
                elif legacy_version == 2: # 1.7.2-dev
                    return 2
                elif legacy_version == 3: # 1.7.2
                    return 3
                elif legacy_version == 4: # 1.8.0-dev
                    return 4
                elif legacy_version == 5: # 1.8.0-dev
                    return 5
                elif legacy_version == 6: # 1.8.0
                    return 9
                elif legacy_version == 7: # 1.9.0-dev
                    return 10
                elif legacy_version == 8: # 1.9.0-dev
                    return 12
                elif legacy_version == 9: # 1.9.0-dev
                    return 12
            return None
        
    def get_available_version(self):
        return self._migratable_db.get_upgrade_version()
        
    def is_versioned(self):
        """ Check if the version metadata has been initiaized."""
        return self._migratable_db.is_versioned()
    
    def initialize_version(self, initial_version):
        """Initialize the version metadata."""
        logging.info("Initializing version metadata to version: '%s'.", initial_version)
        self._migratable_db.version(initial_version)
        
    def initialize(self):
        logging.info("Initializing new database.")
        self._ddbb.create_tables()
        
    def upgrade(self):
        logging.info("Upgrading data from version '%s' to version '%s'.", self.get_version(), self.get_available_version())
        self._ddbb.create_backup()
        pytrainer.upgrade.context.UPGRADE_CONTEXT = self._upgrade_context
        self._migratable_db.upgrade()
        
class DataInitializationException(Exception):
    
    def __init__(self, value):
        self.value = value
        
class DataState(object):
    
    """The state of an installation's data.
    
    The state knows how to update the data to the "current" state."""

    def __init__(self, name, update_function):
        self.name = name
        self._update_function = update_function
        
    def __str__(self):
        return self.name
        
    def update_to_current(self, installed_data):
        """Update the installed data so it is compatible with the current
        version."""
        self._update_function(installed_data)

def _update_fresh(data):
    data.initialize()
    data.initialize_version(data.get_available_version())

def _update_stale(data):
    data.upgrade()

def _update_legacy(data):
    data.initialize_version(data.get_version())
    data.upgrade()

DataState.CURRENT = DataState("CURRENT", lambda data: None)
DataState.FRESH = DataState("FRESH", _update_fresh)
DataState.STALE = DataState("STALE", _update_stale)
DataState.LEGACY = DataState("LEGACY", _update_legacy)

class LegacyVersionProvider(object):
    
    """Provides the DB version number for data not using the current data
    versioning scheme."""
    
    def __init__(self, conf_dir):
        self._conf_dir = conf_dir
        
    def get_legacy_version(self):
        # In versions 1.7.1-1.8.0 the database version was stored as a property
        # in the config file. Versions earlier than 1.7.1 are not supported for
        # upgrades.
        config_file = self._conf_dir + "/conf.xml"
        parser = etree.XMLParser(encoding="UTF8", recover=True)
        xml_tree = etree.parse(config_file, parser=parser)
        config_element = xml_tree.getroot()
        return config_element.get("DB_version")



# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import os, sys
import logging

from lib.xmlUtils import XMLParser

from gui.windowextensions import WindowExtensions

class Extension:
	def __init__(self, data_path = None, parent = None):
		self.data_path=data_path
		self.parent = parent
		self.pytrainer_main = parent
	
	def getActiveExtensions(self):
		retorno = []
		for extension in self.getExtensionList():
			if self.getExtensionInfo(extension[0])[2] == "1":
				retorno.append(extension[0])
		return retorno	
	
	def manageExtensions(self):
		ExtensionList = self.getExtensionList()
		windowextension = WindowExtensions(self.data_path, self)
		windowextension.setList(ExtensionList)
		windowextension.run()

	def getExtensionList(self):
		extensiondir = self.data_path+"/extensions"
		extensionList = []
		for extension in os.listdir(extensiondir):
			extensionxmlfile = extensiondir+"/"+extension+"/conf.xml"
			if os.path.isfile(extensionxmlfile):
				extensioninfo = XMLParser(extensionxmlfile)
				name = extensioninfo.getValue("pytrainer-extension","name")
				description = extensioninfo.getValue("pytrainer-extension","description")
				extensionList.append((extensiondir+"/"+extension,name,description))

		return extensionList
	
	def getExtensionInfo(self,pathExtension):
		info = XMLParser(pathExtension+"/conf.xml")
		name = info.getValue("pytrainer-extension","name")
		description = info.getValue("pytrainer-extension","description")
		code = info.getValue("pytrainer-extension","extensioncode")
		extensiondir = self.pytrainer_main.profile.extensiondir
		helpfile = pathExtension+"/"+info.getValue("pytrainer-extension","helpfile")
		type = info.getValue("pytrainer-extension","type")
	
		if not os.path.isfile(extensiondir+"/"+code+"/conf.xml"):
			status = 0
		else:
			info = XMLParser(extensiondir+"/"+code+"/conf.xml")
			status = info.getValue("pytrainer-extension","status")
		#print name,description,status,helpfile,type
		return name,description,status,helpfile,type

	def getExtensionConfParams(self,pathExtension):
		info = XMLParser(pathExtension+"/conf.xml")
		code = info.getValue("pytrainer-extension","extensioncode")
		extensiondir = self.pytrainer_main.profile.extensiondir
		params = {}
		if not os.path.isfile(extensiondir+"/"+code+"/conf.xml"):
			prefs = info.getAllValues("conf-values")
			prefs.append(("status","0"))
			for pref in prefs:
				params[pref[0]] = info.getValue("pytrainer-extension",pref[0])
		else:
			prefs = info.getAllValues("conf-values")
			prefs.append(("status","0"))
			info = XMLParser(extensiondir+"/"+code+"/conf.xml")
			for pref in prefs:
				params[pref[0]] = info.getValue("pytrainer-extension",pref[0])
				#params.append((pref[0],info.getValue("pytrainer-extension",pref[0])))
		return params

	def setExtensionConfParams(self,pathExtension,savedOptions):
		info = XMLParser(pathExtension+"/conf.xml")
		code = info.getValue("pytrainer-extension","extensioncode")
		extensiondir = self.pytrainer_main.profile.extensiondir+"/"+code
		if not os.path.isdir(extensiondir):
			os.mkdir(extensiondir)
		if not os.path.isfile(extensiondir+"/conf.xml"):
			savedOptions.append(("status","0"))
		info = XMLParser(extensiondir+"/conf.xml")
		info.createXMLFile("pytrainer-extension",savedOptions)

	def loadExtension(self,pathExtension):
		info = XMLParser(pathExtension+"/conf.xml")
		txtbutton = info.getValue("pytrainer-extension","extensionbutton")
		name = info.getValue("pytrainer-extension","name")
		type = info.getValue("pytrainer-extension","type")
		#print "Loading Extension %s" %name
		return txtbutton,pathExtension,type
	

	def getCodeConfValue(self,code,value):
		extensiondir = self.pytrainer_main.profile.extensiondir
		info = XMLParser(extensiondir+"/"+code+"/conf.xml")
		return info.getValue("pytrainer-extension",value)
		
	def importClass(self, pathExtension):
		logging.debug('>>')
		info = XMLParser(pathExtension+"/conf.xml")
	  	#import extension
		extension_dir = os.path.realpath(pathExtension) 
		extension_filename = info.getValue("pytrainer-extension","executable")
		extension_classname = info.getValue("pytrainer-extension","extensioncode")
		extension_type = info.getValue("pytrainer-extension","type")
		options = self.getExtensionConfParams(pathExtension)
		logging.debug("Extension Filename: " + extension_filename )
		logging.debug("Extension Classname: " + extension_classname)
		logging.debug("Extension Type: " + extension_type)
		logging.debug("Extension options: " + str(options))
		sys.path.insert(0, extension_dir)
		module = __import__(extension_filename)
		extensionMain = getattr(module, extension_classname)
		logging.debug('<<')
		return extensionMain(parent=self, pytrainer_main=self.parent, conf_dir=self.pytrainer_main.profile.confdir, options=options)



# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net
# Modified by dgranda

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import os, stat
import logging
from StringIO import StringIO

from lxml import etree
from environment import Environment
from pytrainer.lib.singleton import Singleton
from lib.uc import UC

class Profile(Singleton):
    def __init__(self):
        logging.debug(">>")
        # The first two variables are singletons themselves, profile_options
        # doesn't change, only xml_tree needs to be protected by has_attr.
        self.environment = Environment()
        self.uc = UC()
        #Profile Options and Defaults
        self.profile_options = {
            "prf_name":"default",
            "prf_gender":"",
            "prf_weight":"",
            "prf_height":"",
            "prf_age":"",
            "prf_ddbb":"sqlite",
            "prf_ddbbhost":"",
            "prf_ddbbname":"",
            "prf_ddbbuser":"",
            "prf_ddbbpass":"",
            "version":"0.0",
            "prf_us_system":"False",
            "prf_hrzones_karvonen":"False",
            "prf_maxhr":"",
            "prf_minhr":"",
            "auto_launch_file_selection":"False",
            "import_default_tab":"0",
            "default_viewer":"0",
            "window_size":"800, 640",
            "activitypool_size": "10",
            "prf_startscreen":"current_day",
            }

        if not hasattr(self, 'xml_tree'):
            self.xml_tree = None
            #Parse pytrainer configuration file
            self.refreshConfiguration()

        logging.debug("<<")

    @property
    def data_path(self):
        return self.environment.data_path

    @property
    def tmpdir(self):
        return self.environment.temp_dir

    @property
    def confdir(self):
        return self.environment.conf_dir

    @property
    def config_file(self):
        return self.environment.conf_file

    @property
    def gpxdir(self):
        return self.environment.gpx_dir

    @property
    def extensiondir(self):
        return self.environment.extension_dir

    @property
    def plugindir(self):
        return self.environment.plugin_dir

    def refreshConfiguration(self):
        logging.debug(">>")
        self.configuration = self._parse_config_file(self.config_file)
        logging.debug("Configuration retrieved: %s", str(self.configuration))
        self.uc.set_us(self.prf_us_system)
        self._setZones()
        logging.debug("<<")

    def _setZones(self):
        #maxhr = self.getValue("pytraining","prf_maxhr")
        #resthr = self.getValue("pytraining","prf_minhr")
        try:
            maxhr = int(self.getValue("pytraining","prf_maxhr"))
        except Exception as e:
            logging.error("Failed when retrieving Max Heartrate value: %s" %e)
            logging.info("Setting Max Heartrate to default value: 190")
            maxhr = 190
        try:
            resthr = int(self.getValue("pytraining","prf_minhr"))
        except Exception as e:
            logging.error("Failed when retrieving Min Heartrate value: %s" %e)
            logging.info("Setting Min Heartrate to default value: 65")
            resthr = 65
        self.maxhr = maxhr
        self.rethr = resthr

        if self.getValue("pytraining","prf_hrzones_karvonen")=="True":
            #karvonen method
            targethr1 = ((maxhr - resthr) * 0.50) + resthr
            targethr2 = ((maxhr - resthr) * 0.60) + resthr
            targethr3 = ((maxhr - resthr) * 0.70) + resthr
            targethr4 = ((maxhr - resthr) * 0.80) + resthr
            targethr5 = ((maxhr - resthr) * 0.90) + resthr
            targethr6 = maxhr
        else:
            #not karvonen method
            targethr1 = maxhr * 0.50
            targethr2 = maxhr * 0.60
            targethr3 = maxhr * 0.70
            targethr4 = maxhr * 0.80
            targethr5 = maxhr * 0.90
            targethr6 = maxhr

        self.zone1 = (targethr1,targethr2,"#ffff99",_("Moderate activity"))
        self.zone2 = (targethr2,targethr3,"#ffcc00",_("Weight Control"))
        self.zone3 = (targethr3,targethr4,"#ff9900",_("Aerobic"))
        self.zone4 = (targethr4,targethr5,"#ff6600",_("Anaerobic"))
        self.zone5 = (targethr5,targethr6,"#ff0000",_("VO2 MAX"))
        
    def getMaxHR(self):
        return self.maxhr
        
    def getRestHR(self):
        return self.resthr

    def getZones(self):
        return self.zone5,self.zone4,self.zone3,self.zone2,self.zone1

    def _parse_config_file(self, config_file):
        '''
        Parse the xml configuration file and convert to a dict

        returns: dict with option as key
        '''
        if config_file is None:
            logging.error("Configuration file value not set")
            logging.error("Fatal error, exiting")
            exit(-3)
        elif not os.path.isfile(config_file): #File not found
            logging.error("Configuration '%s' file does not exist" % config_file)
            logging.info("No profile found. Creating default one")
            self.setProfile(self.profile_options)
        elif os.stat(config_file)[stat.ST_SIZE] == 0: #File is empty
            logging.error("Configuration '%s' file is empty" % config_file)
            logging.info("Creating default profile")
            self.setProfile(self.profile_options)
        else:
            logging.debug("Attempting to parse content from %s", config_file)
            parser = etree.XMLParser(encoding='UTF8', recover=True)
            try:
                self.xml_tree = etree.parse(config_file, parser=parser)
            except Exception as e:
                logging.error("Error %s while parsing file %s. Exiting", e, config_file)
                exit(-3)
        #Have a populated xml tree, get pytraining node (root) and convert it to a dict
        pytraining_tag = self.xml_tree.getroot()
        config = {}
        config_needs_update = False
        for key, default in self.profile_options.items():
            value = pytraining_tag.get(key)
            #If property is not found, set it to the default
            if value is None:
                config_needs_update = True
                value = default
            config[key] = value
        #Added a property, so update config
        if config_needs_update:
            self.setProfile(config)
        #Set shorthand var for units of measurement
        self.prf_us_system = True if config["prf_us_system"] == "True" else False
        return config

    def getIntValue(self, tag, variable, default=0):
        ''' Function to return conf value as int
            returns
            -- default if cannot convert to int
            -- None if variable not found
        '''
        result = self.getValue(tag, variable)
        if result is None:
            return None
        try:
            result = int(result)
        except Exception as e:
            logging.debug(str(e))
            result = default
        return result

    def getValue(self, tag, variable):
        if tag != "pytraining":
            print "ERROR - pytraining is the only profile tag supported"
            return None
        elif not self.configuration.has_key(variable):
            return None
        return self.configuration[variable]

    def setValue(self, tag, variable, value, delay_write=False):
        logging.debug(">>")
        if tag != "pytraining":
            logging.error("ERROR: pytraining is the only profile tag supported")
        logging.debug("Setting %s to %s" % (variable, value))
        if self.xml_tree is None:
            #new config file....
            self.xml_tree = etree.parse(StringIO('''<?xml version='1.0' encoding='UTF-8'?><pytraining />'''))
        self.xml_tree.getroot().set(variable, value.decode('utf-8'))
        if not delay_write:
            self.saveProfile()
        logging.debug("<<")

    def setProfile(self,list_options):
        logging.debug(">>")
        for option, value in list_options.items():
            logging.debug("Adding "+option+"|"+value)
            self.setValue("pytraining",option,value,delay_write=True)
        self.uc.set_us(list_options['prf_us_system'])
        logging.debug("<<")

    def saveProfile(self):
        logging.debug("Writting configuration...")
        self.xml_tree.write(self.config_file, xml_declaration=True,
                            encoding='UTF-8')



# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from gui.windowimportdata import WindowImportdata

class Importdata:
	def __init__(self, sport_service, data_path = None, parent = None, config = None):
		self._sport_service = sport_service
		self.data_path=data_path
		self.parent = parent
		self.pytrainer_main = parent
		self.configuration = config
	
	def runImportdata(self):
		windowImportdata = WindowImportdata(self._sport_service, self.data_path, self, self.configuration, self.pytrainer_main)
		windowImportdata.run()



#!/usr/bin/env python

import logging
from math import floor
from osgeo import gdal, gdalnumeric

# from gpxtools
def bilinear_interpolation(tl, tr, bl, br, a, b):
    """
    Based on equation from:
    http://en.wikipedia.org/wiki/Bilinear_interpolation
    
    :Parameters:
        tl : int
            top-left
        tr : int
            top-right
        bl : int
            buttom-left
        br : int
            bottom-right
        a : float
            x distance to top-left
        b : float
            y distance to top-right

    :Returns: (float)
        interpolated value
    """
    b1 = tl
    b2 = bl - tl
    b3 = tr - tl
    b4 = tl - bl - tr + br

    return b1 + b2 * a + b3 * b + b4 * a * b
    

class SrtmTiff(object):
    """
    Provides an interface to SRTM elevation data stored in GeoTIFF file.
    
    Based on code from `eleserver` code by grahamjones139.
    http://code.google.com/p/eleserver/
    """
    tile = {}
    
    def __init__(self, filename):
        """
        Reads the GeoTIFF files into memory ready for processing.
        """
        self.tile = self.load_tile(filename)
    
    def load_tile(self, filename):
        """
        Loads a GeoTIFF tile from disk and returns a dictionary containing
        the file data, plus metadata about the tile.

        The dictionary returned by this function contains the following data:
            xsize - the width of the tile in pixels.
            ysize - the height of the tile in pixels.
            lat_origin - the latitude of the top left pixel in the tile.
            lon_origin - the longitude of the top left pixel in the tile.
            lat_pixel - the height of one pixel in degrees latitude.
            lon_pixel - the width of one pixel in degrees longitude.
            N, S, E, W - the bounding box for this tile in degrees.
            data - a two dimensional array containing the tile data.

        """
        dataset = gdal.Open(filename)
        geotransform = dataset.GetGeoTransform()
        xsize = dataset.RasterXSize
        ysize = dataset.RasterYSize
        lon_origin = geotransform[0]
        lat_origin = geotransform[3]
        lon_pixel = geotransform[1]
        lat_pixel = geotransform[5]
        
        retdict = {
            'xsize': xsize,
            'ysize': ysize,
            'lat_origin': lat_origin,
            'lon_origin': lon_origin,
            'lon_pixel': lon_pixel,
            'lat_pixel': lat_pixel,
            'N': lat_origin,
            'S': lat_origin + lat_pixel*ysize,
            'E': lon_origin + lon_pixel*xsize,
            'W': lon_origin,
            'dataset': dataset,
            }
        
        return retdict  

    def pos_from_lat_lon(self, lat, lon):
        """
        Converts coordinates (lat,lon) into the appropriate (row,column)
        position in the GeoTIFF tile data stored in td.
        """
        td = self.tile
        N = td['N']
        S = td['S']
        E = td['E']
        W = td['W']
        lat_pixel = td['lat_pixel']
        lon_pixel = td['lon_pixel']
        xsize = td['xsize']
        ysize = td['ysize']
        
        rowno_f = (lat-N)/lat_pixel
        colno_f = (lon-W)/lon_pixel
        rowno = int(floor(rowno_f))
        colno = int(floor(colno_f))

        # Error checking to correct any rounding errors.
        if (rowno<0):
            rowno = 0
        if (rowno>(xsize-1)):
            rowno = xsize-1
        if (colno<0):
            colno = 0
        if (colno>(ysize-1)):
            colno = xsize-1
            
        return (rowno, colno, rowno_f, colno_f)
    
    def get_elevation(self, lat, lon):
        """
        Returns the elevation in metres of point (lat, lon).
        
        Uses bilinar interpolation to interpolate the SRTM data to the
        required point.
        """
        row, col, row_f, col_f = self.pos_from_lat_lon(lat, lon)

        # NOTE - THIS IS A FIDDLE TO STOP ERRORS AT THE EDGE OF
        # TILES - IT IS NO CORRECT - WE SHOULD GET TWO POINTS 
        # FROM THE NEXT TILE.
        if row==5999: row=5998
        if col==5999: col=5998

        htarr = gdalnumeric.DatasetReadAsArray(self.tile['dataset'], col, row, 2, 2)
        height = bilinear_interpolation(htarr[0][0], htarr[0][1], htarr[1][0], htarr[1][1],
                                       row_f-row, col_f-col)

        return height



# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import logging
import os
import dateutil.parser
from dateutil.tz import tzlocal

from pytrainer.lib.date import second2time
from pytrainer.lib.gpx import Gpx
from pytrainer.lib.graphdata import GraphData
from pytrainer.environment import Environment
from pytrainer.lib import uc

class ActivityService(object):
    '''
    Class maintains a pool of activities
            size is set at initialisation
    '''
    def __init__(self, pytrainer_main=None, size=1):
        logging.debug(">>")
        #It is an error to try to initialise with no reference to pytrainer_main
        if pytrainer_main is None:
            print("Error - must initialise with a reference to the main pytrainer class")
            return
        self.pytrainer_main = pytrainer_main
        self.max_size = size
        self.pool = {}
        self.pool_queue = []
        logging.debug("Initialising ActivityPool to size: %d" % size)
        logging.debug("<<")

    def clear_pool(self):
        logging.debug(">>")
        logging.debug("Clearing ActivityPool")
        self.pool = {}
        self.pool_queue = []
        logging.debug("<<")

    def remove_activity(self, id):
        sid = str(id)
        if sid in self.pool.keys():
            logging.debug("Found activity in pool")
            self.pool_queue.remove(sid)
            del self.pool[sid]

    def get_activity(self, id):
        sid = str(id)
        if sid in self.pool.keys():
            logging.debug("Found activity in pool")
            #Have accessed this activity, place at end of queue
            self.pool_queue.remove(sid)
            self.pool_queue.append(sid)
        else:
            logging.debug("Activity NOT found in pool")
            self.pool[sid] = Activity(pytrainer_main=self.pytrainer_main, id=id)
            self.pool_queue.append(sid)
        if len(self.pool_queue) > self.max_size:
            sid_to_remove = self.pool_queue.pop(0)
            logging.debug("Removing activity: %s" % sid_to_remove)
            del self.pool[sid_to_remove]
        logging.debug("ActivityPool queue length: %d" % len(self.pool_queue))
        logging.debug("ActivityPool queue: %s" % str(self.pool_queue))
        return self.pool[sid]

class Activity:
    '''
    Class that knows everything about a particular activity

    All values are stored in the class (and DB) in metric and are converted as needed

    tracks                  - (list) tracklist from gpx
    tracklist               - (list of dict) trackpoint data from gpx
    laps                    - (list of dict) lap list
    us_system               - (bool) True: imperial measurement False: metric measurement
    distance_data   - (dict of graphdata classes) contains the graph data with x axis distance
    time_data               - (dict of graphdata classes) contains the graph data with x axis time
    gpx_file                - (string) gpx file name
    gpx                             - (Gpx class) actual gpx instance
    sport_name              - (string) sport name
    sport_id                - (string) id for sport in sports table
    title                   - (string) title of activity
    date                    - (string) date of activity
    time                    - (int) activity duration in seconds
    time_tuple              - (tuple) activity duration as hours, min, secs tuple
    beats                   - (int) average heartrate for activity
    maxbeats                - (int) maximum heartrate for activity
    comments                - (string) activity comments
    calories                - (int) calories of activity
    id                      - (int) id for activity in records table
    date_time_local - (string) date and time of activity in local timezone
    date_time_utc   - (string) date and time of activity in UTC timezone
    date_time               - (datetime) date and time of activity in local timezone
    starttime               - (string)
    distance                - (float) activity distance
    average                 - (float) average speed of activity
    upositive               - (float) height climbed during activity
    unegative               - (float) height decended during activity
    maxspeed                - (float) maximum speed obtained during activity
    maxpace                 - (float) maxium pace obtained during activity
    pace                    - (float) average pace for activity
    has_data                - (bool) true if instance has data populated
    x_axis                  - (string) distance or time, determines what will be graphed on x axis
    x_limits                - (tuple of float) start, end limits of x axis (as determined by matplotlib)
    y1_limits               - (tuple of float) start, end limits of y1 axis (as determined by matplotlib)
    y2_limits               - (tuple of float) start, end limits of y2 axis (as determined by matplotlib)
    x_limits_u              - (tuple of float) start, end limits of x axis (as requested by user)
    y1_limits_u             - (tuple of float) start, end limits of y1 axis (as requested by user)
    y2_limits_u             - (tuple of float) start, end limits of y2 axis (as requested by user)
    show_laps               - (bool) display laps on graphs
    lap_distance    - (graphdata)
    lap_time                - (graphdata)
    pace_limit              - (int) maximum pace that is valid for this activity
    '''
    def __init__(self, pytrainer_main=None, id=None):
        logging.debug(">>")
        self.environment = Environment()
        self.uc = uc.UC()
        self.id = id
        #It is an error to try to initialise with no id
        if self.id is None:
            return
        #It is an error to try to initialise with no reference to pytrainer_main
        if pytrainer_main is None:
            print("Error - must initialise with a reference to the main pytrainer class")
            return
        self.pytrainer_main = pytrainer_main
        self.laps = None
        self.has_data = False
        self._distance_data = {}
        self._time_data = {}
        self._lap_time = None
        self._lap_distance = None
        self.time_pause = 0
        self.pace_limit = None
        self._gpx = None
        self._init_from_db()
        self.x_axis = "distance"
        self.x_limits = (None, None)
        self.y1_limits = (None, None)
        self.y2_limits = (None, None)
        self.x_limits_u = (None, None)
        self.y1_limits_u = (None, None)
        self.y2_limits_u = (None, None)
        self.y1_grid = False
        self.y2_grid = False
        self.x_grid = False
        self.show_laps = False
        logging.debug("<<")

    @property
    def gpx_file(self):
        if self.id:
            filename = "%s/%s.gpx" % (self.environment.gpx_dir, self.id)
            #It is OK to not have a GPX file for an activity - this just limits us to information in the DB
            if os.path.isfile(filename):
                return filename
        logging.debug("No GPX file found for record id: %s", self.id)
        return None

    @property
    def tracks(self):
        if self.gpx:
            return self.gpx.getTrackList()
        else:
            return None

    @property
    def tracklist(self):
        if self.gpx:
            return self.gpx.trkpoints
        else:
            return None

    @property
    def distance_data(self):
        if not self._distance_data:
            self._init_graph_data()
        return self._distance_data

    @property
    def time_data(self):
        if not self._time_data:
            self._init_graph_data()
        return self._time_data

    @property
    def lap_time(self):
        if not self._lap_time:
            self._generate_per_lap_graphs()
        return self._lap_time

    @property
    def lap_distance(self):
        if not self._lap_distance:
            self._generate_per_lap_graphs()
        return self._lap_distance

    @property
    def time_tuple(self):
        return second2time(self.time)

    @property
    def date_time(self):
        if self.date_time_local: #Have a local time stored in DB
            return dateutil.parser.parse(self.date_time_local)
        else: #No local time in DB
            #datetime with localtime offset (using value from OS)
            return dateutil.parser.parse(self.date_time_utc).astimezone(tzlocal())

    @property
    def starttime(self):
        return self.date_time.strftime("%X")

    def __str__(self):
        return '''
tracks (%s)
        tracklist (%s)
        laps (%s)
        us_system (%s)
        distance_data (%s)
        time_data (%s)
        gpx_file (%s)
        gpx (%s)
        sport_name (%s)
        sport_id (%s)
        title (%s)
        date (%s)
        time (%s)
        time_tuple (%s)
        beats (%s)
        maxbeats (%s)
        comments (%s)
        calories (%s)
        id (%s)
        date_time_local (%s)
        date_time_utc (%s)
        date_time (%s)
        starttime (%s)
        distance (%s)
        average (%s)
        upositive (%s)
        unegative (%s)
        maxspeed (%s)
        maxpace (%s)
        pace (%s)
        has_data (%s)
        x_axis (%s)
        x_limits (%s)
        y1_limits (%s)
        y2_limits (%s)
        x_limits_u (%s)
        y1_limits_u (%s)
        y2_limits_u (%s)
        show_laps (%s)
        lap_distance (%s)
        lap_time (%s)
        pace_limit (%s)
''' % ('self.tracks', self.tracklist, self.laps, self.uc.us,
                self.distance_data, self.time_data,
                self.gpx_file, self.gpx, self.sport_name,
                self.sport_id, self.title, self.date, self.time, self.time_tuple, self.beats,
                self.maxbeats, self.comments, self.calories, self.id, self.date_time_local,
                self.date_time_utc, self.date_time, self.starttime, self.distance, self.average,
                self.upositive, self.unegative, self.maxspeed, self.maxpace, self.pace, self.has_data,
                self.x_axis, self.x_limits, self.y1_limits, self.y2_limits, self.x_limits_u, self.y1_limits_u,
                self.y2_limits_u, self.show_laps, self.lap_distance, self.lap_time, self.pace_limit)

    @property
    def gpx(self):
        '''
        Get activity information from the GPX file
        '''
        logging.debug(">>")
        if self._gpx:
            logging.debug("Return pre-created GPX")
            return self._gpx
        elif self.gpx_file:
            logging.debug("Parse GPX")
            #Parse GPX file
            #print "Activity initing GPX.. ",
            self._gpx = Gpx(filename=self.gpx_file) #TODO change GPX code to do less....
            logging.info("GPX Distance: %s | distance (trkpts): %s | duration: %s | duration (trkpts): %s" % (self.gpx.total_dist, self.gpx.total_dist_trkpts, self.gpx.total_time, self.gpx.total_time_trkpts))
            time_diff = self.gpx.total_time_trkpts - self.gpx.total_time
            acceptable_lapse = 4 # number of seconds that duration calculated using lap and trkpts data can differ
            if time_diff > acceptable_lapse:
                self.time_pause = time_diff
                logging.debug("Identified non active time: %s s" % self.time_pause)
            return self._gpx
        else:
            logging.debug("No GPX file found")
            return None
        logging.debug("<<")

    def _init_from_db(self):
        '''
        Get activity information from the DB
        '''
        logging.debug(">>")
        #Get base information
        cols = ("sports.name","id_sports", "date","distance","time","beats","comments","duration",
                                        "average","calories","id_record","title","upositive","unegative",
                                        "maxspeed","maxpace","pace","maxbeats","date_time_utc","date_time_local", "sports.max_pace")
        # outer join on sport id to workaround bug where sport reference is null on records from GPX import
        db_result = self.pytrainer_main.ddbb.select("records left outer join sports on records.sport=sports.id_sports",
                                ", ".join(cols),
                                "id_record=\"%s\" " %self.id)
        if len(db_result) == 1:
            row = db_result[0]
            self.sport_name = row[cols.index('sports.name')]
            if self.sport_name == None:
                self.sport_name = ""
            self.sport_id = row[cols.index('id_sports')]
            self.pace_limit = row[cols.index('sports.max_pace')]
            if self.pace_limit == 0 or self.pace_limit == "":
                self.pace_limit = None
            self.title = row[cols.index('title')]
            if self.title is None:
                self.title = ""
            self.date = row[cols.index('date')]
            self.time = self._int(row[cols.index('time')])
            self.duration = self._int(row[cols.index('duration')])
            self.beats = self._int(row[cols.index('beats')])
            self.comments = row[cols.index('comments')]
            if self.comments is None:
                self.comments = ""
            self.calories = self._int(row[cols.index('calories')])
            self.maxbeats = self._int(row[cols.index('maxbeats')])
            self.date_time_local = row[cols.index('date_time_local')]
            self.date_time_utc = row[cols.index('date_time_utc')]
            self.distance = self._float(row[cols.index('distance')])
            if not self.distance and self.gpx:
                self.distance = self.gpx.total_dist
            self.average = self._float(row[cols.index('average')])
            self.upositive = self._float(row[cols.index('upositive')])
            self.unegative = self._float(row[cols.index('unegative')])
            self.maxspeed = self._float(row[cols.index('maxspeed')])
            self.maxpace = self._float(row[cols.index('maxpace')])
            self.pace = self._float(row[cols.index('pace')])
            self.has_data = True
        else:
            raise Exception("Error - multiple results from DB for id: %s" % self.id)
        #Get lap information
        self.laps = self.pytrainer_main.ddbb.select_dict("laps",
                                ("id_lap", "record", "elapsed_time", "distance", "start_lat", "start_lon", "end_lat", "end_lon", "calories", "lap_number", "intensity", "avg_hr", "max_hr", "max_speed", "laptrigger", "comments"),
                                "record=\"%s\"" % self.id)
        logging.debug("<<")

    def _generate_per_lap_graphs(self):
        '''Build lap based graphs...'''
        logging.debug(">>")
        if self.laps is None:
            logging.debug("No laps to generate graphs from")
            logging.debug("<<")
            return
        #Lap columns
        self._lap_distance = GraphData()
        self._lap_distance.set_color('#CCFF00', '#CCFF00')
        self._lap_distance.graphType = "vspan"
        self._lap_time = GraphData()
        self._lap_time.set_color('#CCFF00', '#CCFF00')
        self._lap_time.graphType = "vspan"
        #Pace
        title = _("Pace by Lap")
        xlabel = "%s (%s)" % (_('Distance'), self.uc.unit_distance)
        ylabel = "%s (%s)" % (_('Pace'), self.uc.unit_pace)
        self.distance_data['pace_lap'] = GraphData(title=title, xlabel=xlabel, ylabel=ylabel)
        self.distance_data['pace_lap'].set_color('#99CCFF', '#99CCFF')
        self.distance_data['pace_lap'].graphType = "bar"
        xlabel=_("Time (seconds)")
        self.time_data['pace_lap'] = GraphData(title=title, xlabel=xlabel, ylabel=ylabel)
        self.time_data['pace_lap'].set_color('#99CCFF', '#99CCFF')
        self.time_data['pace_lap'].graphType = "bar"
        #Speed
        title = _("Speed by Lap")
        xlabel = "%s (%s)" % (_('Distance'), self.uc.unit_distance)
        ylabel = "%s (%s)" % (_('Speed'), self.uc.unit_speed)
        self.distance_data['speed_lap'] = GraphData(title=title, xlabel=xlabel, ylabel=ylabel)
        self.distance_data['speed_lap'].set_color('#336633', '#336633')
        self.distance_data['speed_lap'].graphType = "bar"
        xlabel = _("Time (seconds)")
        self.time_data['speed_lap'] = GraphData(title=title, xlabel=xlabel, ylabel=ylabel)
        self.time_data['speed_lap'].set_color('#336633', '#336633')
        self.time_data['speed_lap'].graphType = "bar"
        for lap in self.laps:
            time = float(lap['elapsed_time'].decode('utf-8')) # time in sql is a unicode string
            dist = lap['distance']/1000 #distance in km
            try:
                pace = time/(60*dist) #min/km
            except ZeroDivisionError:
                pace = 0.0
            try:
                avg_speed = dist/(time/3600) # km/hr
            except:
                avg_speed = 0.0
            if self.pace_limit is not None and pace > self.pace_limit:
                logging.debug("Pace (%s) exceeds limit (%s). Setting to 0" % (str(pace), str(self.pace_limit)))
                pace = 0.0
            logging.debug("Time: %f, Dist: %f, Pace: %f, Speed: %f" % (time, dist, pace, avg_speed))
            self._lap_time.addBars(x=time, y=10)
            self._lap_distance.addBars(x=self.uc.distance(dist), y=10)
            self.distance_data['pace_lap'].addBars(x=self.uc.distance(dist), y=pacekm2miles(pace))
            self.time_data['pace_lap'].addBars(x=time, y=self.uc.speed(pace))
            self.distance_data['speed_lap'].addBars(x=self.uc.distance(dist), y=self.uc.speed(avg_speed))
            self.time_data['speed_lap'].addBars(x=time, y=self.uc.speed(avg_speed))
        logging.debug("<<")

    def _init_graph_data(self):
        logging.debug(">>")
        if self.tracklist is None:
            logging.debug("No tracklist in activity")
            logging.debug("<<")
            return
        #Profile
        title = _("Elevation")
        xlabel = "%s (%s)" % (_('Distance'), self.uc.unit_distance)
        ylabel = "%s (%s)" % (_('Elevation'), self.uc.unit_height)
        self._distance_data['elevation'] = GraphData(title=title, xlabel=xlabel, ylabel=ylabel)
        self._distance_data['elevation'].set_color('#ff0000', '#ff0000')
        self._distance_data['elevation'].show_on_y1 = True #Make graph show elevation by default
        xlabel = _("Time (seconds)")
        self._time_data['elevation'] = GraphData(title=title,xlabel=xlabel, ylabel=ylabel)
        self._time_data['elevation'].set_color('#ff0000', '#ff0000')
        self._time_data['elevation'].show_on_y1 = True #Make graph show elevation by default
        #Corrected Elevation...
        title = _("Corrected Elevation")
        xlabel = "%s (%s)" % (_('Distance'), self.uc.unit_distance)
        ylabel = "%s (%s)" % (_('Corrected Elevation'), self.uc.unit_height)
        self._distance_data['cor_elevation'] = GraphData(title=title, xlabel=xlabel, ylabel=ylabel)
        self._distance_data['cor_elevation'].set_color('#993333', '#993333')
        xlabel=_("Time (seconds)")
        self._time_data['cor_elevation'] = GraphData(title=title,xlabel=xlabel, ylabel=ylabel)
        self._time_data['cor_elevation'].set_color('#993333', '#993333')
        #Speed
        title = _("Speed")
        xlabel = "%s (%s)" % (_('Distance'), self.uc.unit_distance)
        ylabel = "%s (%s)" % (_('Speed'), self.uc.unit_speed)
        self._distance_data['speed'] = GraphData(title=title, xlabel=xlabel, ylabel=ylabel)
        self._distance_data['speed'].set_color('#000000', '#000000')
        xlabel = _("Time (seconds)")
        self._time_data['speed'] = GraphData(title=title,xlabel=xlabel, ylabel=ylabel)
        self._time_data['speed'].set_color('#000000', '#000000')
        #Pace
        title = _("Pace")
        xlabel = "%s (%s)" % (_('Distance'), self.uc.unit_distance)
        ylabel = "%s (%s)" % (_('Pace'), self.uc.unit_pace)
        self._distance_data['pace'] = GraphData(title=title, xlabel=xlabel, ylabel=ylabel)
        self._distance_data['pace'].set_color('#0000ff', '#0000ff')
        xlabel = _("Time (seconds)")
        self._time_data['pace'] = GraphData(title=title,xlabel=xlabel, ylabel=ylabel)
        self._time_data['pace'].set_color('#0000ff', '#0000ff')
        #Heartrate
        title = _("Heart Rate")
        xlabel = "%s (%s)" % (_('Distance'), self.uc.unit_distance)
        ylabel = "%s (%s)" % (_('Heart Rate'), _('bpm'))
        self._distance_data['hr'] = GraphData(title=title, xlabel=xlabel, ylabel=ylabel)
        self._distance_data['hr'].set_color('#00ff00', '#00ff00')
        xlabel = _("Time (seconds)")
        self._time_data['hr'] = GraphData(title=title,xlabel=xlabel, ylabel=ylabel)
        self._time_data['hr'].set_color('#00ff00', '#00ff00')
        #Heartrate as %
        maxhr = self.pytrainer_main.profile.getMaxHR()
        title = _("Heart Rate (% of max)")
        xlabel = "%s (%s)" % (_('Distance'), self.uc.unit_distance)
        ylabel = "%s (%s)" % (_('Heart Rate'), _('%'))
        self._distance_data['hr_p'] = GraphData(title=title, xlabel=xlabel, ylabel=ylabel)
        self._distance_data['hr_p'].set_color('#00ff00', '#00ff00')
        xlabel = _("Time (seconds)")
        self._time_data['hr_p'] = GraphData(title=title, xlabel=xlabel, ylabel=ylabel)
        self._time_data['hr_p'].set_color('#00ff00', '#00ff00')
        #Cadence
        title = _("Cadence")
        xlabel = "%s (%s)" % (_('Distance'), self.uc.unit_distance)
        ylabel = "%s (%s)" % (_('Cadence'), _('rpm'))
        self._distance_data['cadence'] = GraphData(title=title, xlabel=xlabel, ylabel=ylabel)
        self._distance_data['cadence'].set_color('#cc00ff', '#cc00ff')
        xlabel = _("Time (seconds)")
        self._time_data['cadence'] = GraphData(title=title, xlabel=xlabel, ylabel=ylabel)
        self._time_data['cadence'].set_color('#cc00ff', '#cc00ff')
        for track in self.tracklist:
            try:
                pace = 60/track['velocity']
                if self.pace_limit is not None and pace > self.pace_limit:
                    logging.debug("Pace (%s) exceeds limit (%s). Setting to 0" % (str(pace), str(self.pace_limit)))
                    pace = 0  #TODO this should be None when we move to newgraph...
            except Exception as e:
                #print type(e), e
                pace = 0
            try:
                hr_p = float(track['hr'])/maxhr*100
            except:
                hr_p = 0
            self._distance_data['elevation'].addPoints(x=self.uc.distance(track['elapsed_distance']),
                                                       y=self.uc.height(track['ele']))
            self._distance_data['cor_elevation'].addPoints(x=self.uc.distance(track['elapsed_distance']),
                                                           y=self.uc.height(track['correctedElevation']))
            self._distance_data['speed'].addPoints(x=self.uc.distance(track['elapsed_distance']),
                                                   y=self.uc.speed(track['velocity']))
            self._distance_data['pace'].addPoints(x=self.uc.distance(track['elapsed_distance']),
                                                  y=self.uc.distance(pace))
            self._distance_data['hr'].addPoints(x=self.uc.distance(track['elapsed_distance']),
                                                y=track['hr'])
            self._distance_data['hr_p'].addPoints(x=self.uc.distance(track['elapsed_distance']),
                                                  y=hr_p)
            self._distance_data['cadence'].addPoints(x=self.uc.distance(track['elapsed_distance']),
                                                     y=track['cadence'])
            self._time_data['elevation'].addPoints(x=track['time_elapsed'],
                                                   y=self.uc.height(track['ele']))
            self._time_data['cor_elevation'].addPoints(x=track['time_elapsed'],
                                                       y=self.uc.height(track['correctedElevation']))
            self._time_data['speed'].addPoints(x=track['time_elapsed'],
                                               y=self.uc.speed(track['velocity']))
            self._time_data['pace'].addPoints(x=track['time_elapsed'],
                                              y=self.uc.distance(pace))
            self._time_data['hr'].addPoints(x=track['time_elapsed'], y=track['hr'])
            self._time_data['hr_p'].addPoints(x=track['time_elapsed'], y=hr_p)
            self._time_data['cadence'].addPoints(x=track['time_elapsed'], y=track['cadence'])
        #Remove data with no values
        for item in self._distance_data.keys():
            if len(self._distance_data[item]) == 0:
                logging.debug( "No values for %s. Removing...." % item )
                del self._distance_data[item]
        for item in self._time_data.keys():
            if len(self._time_data[item]) == 0:
                logging.debug( "No values for %s. Removing...." % item )
                del self._time_data[item]
        logging.debug("<<")
        #Add Heartrate zones graphs
        if 'hr' in self._distance_data:
            zones = self.pytrainer_main.profile.getZones()
            title = _("Heart Rate zone")
            xlabel = "%s (%s)" % (_('Distance'), self.uc.unit_distance)
            ylabel = "%s (%s)" % (_('Heart Rate'), _('bpm'))
            self._distance_data['hr_z'] = GraphData(title=title, xlabel=xlabel, ylabel=ylabel)
            self._distance_data['hr_z'].graphType = "hspan"
            self._distance_data['hr_z'].set_color(None, None)
            xlabel = _("Time (seconds)")
            self._time_data['hr_z'] = GraphData(title=title,xlabel=xlabel, ylabel=ylabel)
            self._time_data['hr_z'].set_color(None, None)
            for zone in zones:
                self._distance_data['hr_z'].addPoints(x=zone[0], y=zone[1], label=zone[3], color=zone[2])
                self._time_data['hr_z'].addPoints(x=zone[0], y=zone[1], label=zone[3], color=zone[2])

    def _float(self, value):
        try:
            result = float(value)
        except:
            result = 0.0
        return result

    def _int(self, value):
        try:
            result = int(value)
        except:
            result = 0
        return result

    def get_value_f(self, param, format=None):
        ''' Function to return a value formated as a string
                - takes into account US/metric
                - also appends units if required
        '''
        value = self.get_value(param)
        if not value:
            #Return blank string if value is None or 0
            return ""
        if format is not None:
            result = format % value
        else:
            result = str(value)
        return result

    def get_value(self, param):
        ''' Function to get the value of various params in this activity instance
                Automatically returns values converted to imperial if needed
        '''
        if param == 'distance':
            return self.uc.distance(self.distance)
        elif param == 'average':
            return self.uc.speed(self.average)
        elif param == 'upositive':
            return self.uc.height(self.upositive)
        elif param == 'unegative':
            return self.uc.height(self.unegative)
        elif param == 'maxspeed':
            return self.uc.speed(self.maxspeed)
        elif param == 'maxpace':
            return uc.float2pace(self.uc.pace(self.maxpace))
        elif param == 'pace':
            return uc.float2pace(self.uc.pace(self.pace))
        elif param == 'calories':
            return self.calories
        elif param == 'time':
            if not self.time:
                return ""
            _hour ,_min, _sec = second2time(self.time)
            if _hour == 0:
                return "%02d:%02d" % (_min, _sec)
            else:
                return "%0d:%02d:%02d" % (_hour, _min, _sec)
        else:
            print "Unable to provide value for unknown parameter (%s) for activity" % param
            return None



# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net
# Modified by dgranda

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# Added to support python installations older than 2.6
from __future__ import with_statement

import logging
import os
import StringIO
from lxml import etree
from pytrainer.lib.date import getDateTime

class garmintools():
	def __init__(self, parent = None, data_path = None):
		self.parent = parent
		self.pytrainer_main = parent.parent
		self.tmpdir = self.pytrainer_main.profile.tmpdir
		self.main_data_path = data_path
		self.data_path = os.path.dirname(__file__)
		self.xmldoc = None
		self.activitiesSummary = []

	def getXmldoc(self):
		''' Function to return parsed xmlfile '''
		return self.xmldoc

	def getFileType(self):
		return _("Garmin tools dump file")

	def getActivitiesSummary(self):
		return self.activitiesSummary

	def testFile(self, filename):
		logging.debug('>>')
		logging.debug("Testing " + filename)
		#Check if file is a garmintools dump
		try:
			with open(filename, 'r') as f:
				xmlString = f.read()
			#add a root element to make correct xml
			fileString = StringIO.StringIO("<root>"+xmlString+"</root>")
			#parse string as xml
			xmldoc = etree.parse(fileString)
			#Parse XML schema
			xmlschema_doc = etree.parse(self.main_data_path+"schemas/garmintools.xsd")
			xmlschema = etree.XMLSchema(xmlschema_doc)
			if (xmlschema.validate(xmldoc)):
				#Valid garmintools file
				self.xmldoc = xmldoc
				startTime = getDateTime(self.startTimeFromFile(xmldoc))
				indatabase = self.inDatabase(xmldoc, startTime)
				sport = self.getSport(xmldoc)
				distance, duration  = self.getDetails(xmldoc, startTime)
				distance = distance / 1000.0
				self.activitiesSummary.append( (0,
												indatabase,
												startTime[1].strftime("%Y-%m-%dT%H:%M:%S"),
												"%0.2f" % distance ,
												str(duration),
												sport,
												) )
				return True
		except:
			#Not garmintools dump file
			return False
		return False

	def inDatabase(self, tree, startTime):
		#comparing date and start time (sport may have been changed in DB after import)
		time = startTime
		if time is None:
			return False
		time = time[0].strftime("%Y-%m-%dT%H:%M:%SZ")
		if self.parent.parent.ddbb.select("records","*","date_time_utc=\"%s\"" % (time)):
			return True
		else:
			return False

	def getDetails(self, tree, startTime):
		root = tree.getroot()
		points = root.findall(".//point")
		while True:
			lastPoint = points[-1]
			try:
				distance = lastPoint.get("distance")
				if distance is None:
					points = points[:-1]
					continue
				time = lastPoint.get("time")
				break
			except:
				points = points[:-1]
				continue
		return float(distance), getDateTime(time)[0]-startTime[0]

	def getSport(self, tree):
		#return sport from file
		root = tree.getroot()
		sportElement = root.find(".//run")
		try:
			sport = sportElement.get("sport")
			sport = sport.capitalize()
		except:
			sport = "import"
		return sport

	def startTimeFromFile(self, tree):
		root = tree.getroot()
		#Find first point
		pointElement = root.find(".//point")
		if pointElement is not None:
			#Try to get time from point
			time = pointElement.get("time")
			print "#TODO first time is different from time used by gpsbabel and has locale embedded: " + time
			return time
		return None

	def getGPXFile(self, ID, file_id):
		"""
			Generate GPX file based on activity ID

			Returns (sport, GPX filename)
		"""
		sport = None
		gpxFile = None
		if ID == "0": #Only one activity in file
			gpxFile = "%s/garmintools-%s-%s.gpx" % (self.tmpdir, file_id, ID)
			sport = self.getSport(self.xmldoc)
			self.createGPXfile(gpxFile, self.xmldoc)
		return sport, gpxFile

	def createGPXfile(self, gpxfile, tree):
		""" Function to transform a Garmintools dump file to a valid GPX+ file
		"""
		xslt_doc = etree.parse(self.data_path+"/translate_garmintools.xsl")
		transform = etree.XSLT(xslt_doc)
		result_tree = transform(tree)
		result_tree.write(gpxfile, xml_declaration=True, encoding='UTF-8')




# -*- coding: iso-8859-1 -*-

#Copyright (C) Nathan Jones ncjones@users.sourceforge.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import gtk
from pytrainer.core.equipment import Equipment

class EquipmentStore(gtk.ListStore):
    
    def __init__(self, equipment_service):
        super(EquipmentStore, self).__init__(int, str, float, str, bool)
        self._equipment_service = equipment_service
        for equipment in equipment_service.get_all_equipment():
            self._append_row(equipment)
        self.set_default_sort_func(self._sort)
        self.set_sort_column_id(-1, gtk.SORT_ASCENDING)
        
    def _sort(self, store, x, y):
        if self.get_value(x, 4) != self.get_value(y, 4):
            return self.get_value(y, 4)-self.get_value(x, 4)
        elif self.get_value(x, 2) == self.get_value(y, 2):
            return 0
        else:
            return self.get_value(y, 2)*100-self.get_value(x, 2)*100
    
    def _append_row(self, equipment):
        self.append(self._create_tuple(equipment))
        
    def _create_tuple(self, equipment):
        usage = self._equipment_service.get_equipment_usage(equipment) + equipment.prior_usage
        return (equipment.id,
                equipment.description,
                self._calculate_usage_percent(usage, equipment.life_expectancy),
                str(int(round(usage)))  + " / " + str(equipment.life_expectancy),
                equipment.active)
        
    def _calculate_usage_percent(self, usage, life_expectancy):
        if life_expectancy == 0:
            return 0 
        else:
            return min(100, 100.0 * usage / life_expectancy) 
        
    def add_equipment(self, equipment):
        added_equipment = self._equipment_service.store_equipment(equipment)
        self._append_row(added_equipment)
        
    def get_equipment_item(self, item_path):
        item = None
        if item_path is not None:
            item_id = self.get_value(self.get_iter(item_path), 0)
            item = self._equipment_service.get_equipment_item(item_id)
        return item
        
    def edit_equipment(self, item_path, equipment):
        updated_item = self._equipment_service.store_equipment(equipment)
        for (column_index, value) in enumerate(self._create_tuple(updated_item)):
            self.set(self.get_iter(item_path), column_index, value)
        
    def remove_equipment(self, item_path):
        item = self.get_equipment_item(item_path)
        self._equipment_service.remove_equipment(item)
        self.remove(self.get_iter(item_path))

class EquipmentUi(gtk.HBox):
    
    def __init__(self, glade_conf_dir, equipment_service):
        gtk.HBox.__init__(self)
        self._equipment_store = EquipmentStore(equipment_service)
        self._builder = gtk.Builder()
        self._builder.add_from_file(glade_conf_dir + "/equipment.ui")
        self._init_tree_view()
        self._init_signals()
        self.add(self._get_notebook())
        self.set_property('visible', True)
        
    def _get_notebook(self):
        return self._builder.get_object("notebookEquipment")
    
    def _get_tree_view(self):
        return self._builder.get_object("treeviewEquipmentList")

    def _init_tree_view(self):
        tree_view = self._get_tree_view()
        column = gtk.TreeViewColumn(_("Description"), gtk.CellRendererText(), text=1)
        column.set_resizable(True)
        tree_view.append_column(column)
        tree_view.append_column(gtk.TreeViewColumn(_("Usage"), gtk.CellRendererProgress(), value=2, text=3))
        tree_view.append_column(gtk.TreeViewColumn(_("Active"), gtk.CellRendererToggle(), active=4))
        # add filler column
        tree_view.append_column(gtk.TreeViewColumn())
        tree_view.set_model(self._equipment_store)
        
    def _init_signals(self):
        self._builder.connect_signals({
            "add_equipment_clicked": self._add_equipment_clicked,
            "cancel_add_equipment_clicked": self._cancel_add_equipment_clicked,
            "confirm_add_equipment_clicked": self._confirm_add_equipment_clicked,
            "equipment_cursor_changed": self._equipment_cursor_changed,
            "edit_equipment_clicked": self._edit_equipment_clicked,
            "equipment_row_activated": self._edit_equipment_clicked,
            "cancel_edit_equipment_clicked": self._cancel_edit_equipment_clicked,
            "confirm_edit_equipment_clicked": self._confirm_edit_equipment_clicked,
            "delete_equipment_clicked": self._delete_equipment_clicked,
            "cancel_delete_equipment_clicked": self._cancel_delete_equipment_clicked,
            "confirm_delete_equipment_clicked": self._confirm_delete_equipment_clicked})
        
    def _get_selected_equipment_path(self):
        (path, _) = self._get_tree_view().get_cursor()
        return path
    
    def _get_selected_equipment_item(self):
        return self._equipment_store.get_equipment_item(self._get_selected_equipment_path())
    
    def clear_add_equipment_fields(self):
        self._builder.get_object("entryEquipmentAddDescription").set_text("") 
        self._builder.get_object("entryEquipmentAddLifeExpectancy").set_text("0")
        self._builder.get_object("entryEquipmentAddPriorUsage").set_text("0")
        self._builder.get_object("checkbuttonEquipmentAddActive").set_active(True)
        self._builder.get_object("textviewEquipmentAddNotes").get_buffer().set_text("")
        
    def show_page_equipment_list(self):
        self._get_notebook().set_current_page(0)
        
    def show_page_equipment_add(self):
        self._get_notebook().set_current_page(1)
        self._builder.get_object("entryEquipmentAddDescription").grab_focus()
        
    def show_page_equipment_edit(self):
        self._get_notebook().set_current_page(2)
        
    def show_page_equipment_delete(self):
        self._get_notebook().set_current_page(3)
    
    def _add_equipment_clicked(self, widget):
        self.clear_add_equipment_fields()
        self.show_page_equipment_add()
    
    def _cancel_add_equipment_clicked(self, widget):
        self.show_page_equipment_list()
    
    def _confirm_add_equipment_clicked(self, widget):
        #FIXME input validation for numeric fields
        description = self._builder.get_object("entryEquipmentAddDescription").get_text()
        life_expectancy = self._builder.get_object("entryEquipmentAddLifeExpectancy").get_text()
        prior_usage = self._builder.get_object("entryEquipmentAddPriorUsage").get_text()
        active = self._builder.get_object("checkbuttonEquipmentAddActive").get_active()
        notes_buffer = self._builder.get_object("textviewEquipmentAddNotes").get_buffer()
        notes = notes_buffer.get_text(notes_buffer.get_start_iter(), notes_buffer.get_end_iter())
        new_equipment = Equipment()
        new_equipment.description = unicode(description)
        new_equipment.active = active
        new_equipment.life_expectancy = life_expectancy
        new_equipment.prior_usage = prior_usage
        new_equipment.notes = unicode(notes)
        self._equipment_store.add_equipment(new_equipment)
        self.show_page_equipment_list()
        
    def _equipment_cursor_changed(self, widget, *args):
        item_selected = self._get_selected_equipment_item() != None
        self._builder.get_object("buttonEquipmentEdit").set_sensitive(item_selected)
        self._builder.get_object("buttonEquipmentDelete").set_sensitive(item_selected)
    
    def _edit_equipment_clicked(self, widget, *args):
        item = self._get_selected_equipment_item()
        self._builder.get_object("entryEquipmentEditDescription").set_text(item.description)
        self._builder.get_object("entryEquipmentEditLifeExpectancy").set_text(str(item.life_expectancy))
        self._builder.get_object("entryEquipmentEditPriorUsage").set_text(str(item.prior_usage))
        self._builder.get_object("checkbuttonEquipmentEditActive").set_active(item.active)
        if item.notes != None:
            self._builder.get_object("textviewEquipmentEditNotes").get_buffer().set_text(item.notes)
        self.show_page_equipment_edit()
        
    def _cancel_edit_equipment_clicked(self, widget):
        self.show_page_equipment_list()
        
    def _confirm_edit_equipment_clicked(self, widget):
        item = self._get_selected_equipment_item()
        description_text = self._builder.get_object("entryEquipmentEditDescription").get_text()
        item.description = unicode(description_text)
        item.life_expectancy = self._builder.get_object("entryEquipmentEditLifeExpectancy").get_text()
        item.prior_usage = self._builder.get_object("entryEquipmentEditPriorUsage").get_text()
        item.active = self._builder.get_object("checkbuttonEquipmentEditActive").get_active()
        notes_buffer = self._builder.get_object("textviewEquipmentEditNotes").get_buffer()
        notes_text = notes_buffer.get_text(notes_buffer.get_start_iter(), notes_buffer.get_end_iter())
        item.notes = unicode(notes_text)
        self._equipment_store.edit_equipment(self._get_selected_equipment_path(), item)
        self.show_page_equipment_list()
    
    def _delete_equipment_clicked(self, widget):
        self.show_page_equipment_delete()
    
    def _cancel_delete_equipment_clicked(self, widget):
        self.show_page_equipment_list()
    
    def _confirm_delete_equipment_clicked(self, widget):
        self._equipment_store.remove_equipment(self._get_selected_equipment_path())
        self.show_page_equipment_list()



# -*- coding: utf-8 -*-

#Copyright (C) Nathan Jones ncjones@users.sourceforge.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import unittest
from pytrainer.core.sport import Sport, SportService, SportServiceException
import mock
from pytrainer.lib.sqliteUtils import Sql
import pytrainer.core
from pytrainer.lib.ddbb import DDBB

class SportTest(unittest.TestCase):
    
    def test_id_should_default_to_none(self):
        sport = Sport()
        self.assertEquals(None, sport.id)
        
    def test_id_should_accept_integer(self):
        sport = Sport()
        sport.id = 1
        self.assertEquals(1, sport.id)
        
    def test_id_should_accept_integer_string(self):
        sport = Sport()
        sport.id = "1"
        self.assertEquals(1, sport.id)
        
    def test_id_should_not_accept_non_integer_string(self):
        sport = Sport()
        try:
            sport.id = "1.1"
        except(ValueError):
            pass
        else:
            self.fail()
            
    def test_id_should_not_accept_none(self):
        sport = Sport()
        try:
            sport.id = None
        except(TypeError):
            pass
        else:
            self.fail()
        
    def test_name_should_default_to_empty_string(self):
        sport = Sport()
        self.assertEquals(u"", sport.name)
        
    def test_name_should_accept_unicode_string(self):
        sport = Sport()
        sport.name = u"Unicycling"
        self.assertEquals(u"Unicycling", sport.name)
        
    def test_name_should_not_accept_non_unicode_string(self):
        sport = Sport()
        try:
            sport.name = "Juggling"
        except(TypeError):
            pass
        else:
            self.fail()
            
    def test_name_should_not_accept_none(self):
        sport = Sport()
        try:
            sport.name = None
        except(TypeError):
            pass
        else:
            self.fail()
            
    def test_met_should_default_to_None(self):
        sport = Sport()
        self.assertEquals(None, sport.met)
        
    def test_met_should_accept_float(self):
        sport = Sport()
        sport.met = 22.5
        self.assertEquals(22.5, sport.met)
        
    def test_met_should_accept_float_string(self):
        sport = Sport()
        sport.met = "22.5"
        self.assertEquals(22.5, sport.met)
        
    def test_met_should_not_accept_non_float_string(self):
        sport = Sport()
        try:
            sport.met = "22.5kg"
        except(ValueError):
            pass
        else:
            self.fail()
                     
    def test_met_should_not_accept_negative_value(self):
        sport = Sport()
        try:
            sport.met = -1
        except(ValueError):
            pass
        else:
            self.fail()
            
    def test_met_should_accept_none(self):
        sport = Sport()
        sport.met = None
        self.assertEquals(None, sport.met)
            
    def test_weight_should_default_to_zero(self):
        sport = Sport()
        self.assertEquals(0, sport.weight)
        
    def test_weight_should_accept_float(self):
        sport = Sport()
        sport.weight = 22.5
        self.assertEquals(22.5, sport.weight)
        
    def test_weight_should_accept_float_string(self):
        sport = Sport()
        sport.weight = "22.5"
        self.assertEquals(22.5, sport.weight)
        
    def test_weight_should_not_accept_non_float_string(self):
        sport = Sport()
        try:
            sport.weight = "22.5kg"
        except(ValueError):
            pass
        else:
            self.fail()
            
    def test_weight_should_not_accept_negative_value(self):
        sport = Sport()
        try:
            sport.weight = -1
        except(ValueError):
            pass
        else:
            self.fail()
            
    def test_weight_should_not_accept_none(self):
        sport = Sport()
        try:
            sport.weight = None
        except(TypeError):
            pass
        else:
            self.fail()
            
    def test_max_pace_should_default_to_none(self):
        sport = Sport()
        self.assertEquals(None, sport.max_pace)
        
    def test_max_pace_should_accept_integer(self):
        sport = Sport()
        sport.max_pace = 220
        self.assertEquals(220, sport.max_pace)
        
    def test_max_pace_should_accept_integer_string(self):
        sport = Sport()
        sport.max_pace = "220"
        self.assertEquals(220, sport.max_pace)
        
    def test_max_pace_should_not_accept_non_integer_string(self):
        sport = Sport()
        try:
            sport.max_pace = "225s"
        except(ValueError):
            pass
        else:
            self.fail()
        
    def test_max_pace_should_take_floor_of_float(self):
        sport = Sport()
        sport.max_pace = 220.6
        self.assertEquals(220, sport.max_pace)
            
    def test_max_pace_should_not_accept_negative_value(self):
        sport = Sport()
        try:
            sport.max_pace = -1
        except(ValueError):
            pass
        else:
            self.fail()
            
    def test_max_pace_should_accept_none(self):
        sport = Sport()
        sport.max_pace = None
        self.assertEquals(None, sport.max_pace)
        
    def test_color_should_default_to_blue(self):
        sport = Sport()
        self.assertEquals(0x0000ff, sport.color.rgb_val)
        
    def test_color_should_not_accept_none(self):
        sport = Sport()
        try:
            sport.color = None
        except(ValueError):
            pass
        else:
            self.fail()


class SportServiceTest(unittest.TestCase):
    
    def setUp(self):
        profile = mock.Mock()
        profile.getValue = mock.Mock(return_value='memory')
        self.mock_ddbb = DDBB(profile)
        self.mock_ddbb.connect()
        self.mock_ddbb.create_tables(add_default=False)
        self.sport_service = SportService(self.mock_ddbb)

    def tearDown(self):
        self.mock_ddbb.disconnect()
        
    def test_store_sport_should_insert_row_when_sport_has_no_id(self):
        sport = Sport()
        sport.name = u"Test name"
        sport = self.sport_service.store_sport(sport)
        self.assertEquals(1, sport.id)

    
    def test_store_sport_should_update_row_when_sport_has_id(self):
        sport = Sport()
        sport.name = u"Test name"
        sport = self.sport_service.store_sport(sport)
        sport.name = u"New name"
        self.sport_service.store_sport(sport)
        sport = self.sport_service.get_sport(1)
        self.assertEquals(sport.name, u"New name")
        
    def test_store_sport_should_return_stored_sport(self):
        sport = Sport()
        stored_sport = self.sport_service.store_sport(sport)
        self.assertEquals(1, stored_sport.id)
    
    def test_store_sport_should_error_when_sport_has_unknown_id(self):
        sport = Sport()
        sport.id = 100
        try:
            self.sport_service.store_sport(sport)
        except(SportServiceException):
            pass
        else:
            self.fail()
            
    def test_store_sport_should_error_when_new_sport_has_duplicate_name(self):
        sport1 = Sport()
        sport1.name = u"Test name"
        self.sport_service.store_sport(sport1)
        sport2 = Sport()
        sport2.name = u"Test name"
        try:
            self.sport_service.store_sport(sport2)
        except(SportServiceException):
            pass
        else:
            self.fail()

    def test_store_sport_should_error_when_existing_sport_has_duplicate_name(self):
        sport1 = Sport()
        sport1.name = u"Test name"
        self.sport_service.store_sport(sport1)
        sport2 = Sport()
        sport2.name = u"New name"
        self.sport_service.store_sport(sport2)
        sport1.name = u"New name"
        try:
            self.sport_service.store_sport(sport1)
        except(SportServiceException):
            pass
        else:
            self.fail()
    
    def test_get_sport_returns_none_for_nonexistant_sport(self):
        sport = self.sport_service.get_sport(1)
        self.assertEquals(None, sport)
        
    def test_get_sport_returns_sport_with_id(self):
        sport = Sport()
        sport.name = u"Test name"
        self.sport_service.store_sport(sport)
        sport = self.sport_service.get_sport(1)
        self.assertEquals(1, sport.id)
        
    def test_get_sport_raises_error_for_id_none(self):
        try:
            self.sport_service.get_sport(None)
        except(ValueError):
            pass
        else:
            self.fail()
        
    def test_get_sport_by_name_returns_none_for_nonexistant_sport(self):
        sport = self.sport_service.get_sport_by_name("no such sport")
        self.assertEquals(None, sport)
        
    def test_get_sport_by_name_returns_sport_with_name(self):
        sport1 = Sport()
        sport1.name = u"rugby"
        self.sport_service.store_sport(sport1)
        sport2 = self.sport_service.get_sport_by_name("rugby")
        self.assertEquals(u"rugby", sport2.name)
        
    def test_get_sport_by_name_raises_error_for_none_sport_name(self):
        try:
            self.sport_service.get_sport_by_name(None)
        except(ValueError):
            pass
        else:
            self.fail()
        
    def test_get_all_sports_should_return_all_sports_in_query_result(self):
        sport1 = Sport()
        sport1.name = u"Test name"
        self.sport_service.store_sport(sport1)
        sport2 = Sport()
        sport2.name = u"Test name 2"
        self.sport_service.store_sport(sport2)
        sports = self.sport_service.get_all_sports()
        self.assertEquals(2, len(sports))
        sport1 = sports[0]
        self.assertEquals(1, sport1.id)
        sport2 = sports[1]
        self.assertEquals(2, sport2.id)
    
    def test_get_all_sports_should_return_no_sports_when_query_result_empty(self):
        sports = self.sport_service.get_all_sports()
        for i in sports:
            print i.name
        self.assertEquals(0, len(sports))
        
    def test_remove_sport_should_error_when_sport_has_no_id(self):
        sport = Sport()
        try:
            self.sport_service.remove_sport(sport)
        except(SportServiceException):
            pass
        else:
            self.fail()
        
    def test_remove_sport_should_error_when_sport_has_unknown_id(self):
        sport = Sport()
        sport.id = 100
        try:
            self.sport_service.remove_sport(sport)
        except(SportServiceException):
            pass
        else:
            self.fail()
            
    def test_remove_sport_should_remove_associated_entries(self):
        sport = Sport()
        sport.name = u"Test name"
        sport = self.sport_service.store_sport(sport)
        self.sport_service.remove_sport(sport)
        result = self.sport_service.get_sport(1)
        self.assertEquals(result, None)



# -*- coding: utf-8 -*-

#Copyright (C) Nathan Jones ncjones@users.sourceforge.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import unittest
import mock
from pytrainer.core.equipment import Equipment, EquipmentService,\
    EquipmentServiceException
from pytrainer.lib.ddbb import DDBB

class EquipmentTest(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass
    
    def test_id_defaults_to_none(self):
        equipment = Equipment()
        self.assertEqual(None, equipment.id)
            
    def test_id_set_to_integer(self):
        equipment = Equipment()
        equipment.id = 2
        self.assertEquals(2, equipment.id)
            
    def test_id_set_to_numeric_string(self):
        equipment = Equipment()
        equipment.id = "3"
        self.assertEquals(3, equipment.id)

    def test_id_set_to_non_numeric_string(self):
        equipment = Equipment()
        try:
            equipment.id = "test"
        except(ValueError):
            pass
        else:
            self.fail("Should not be able to set equipment id to non numeric value.")
            
    def test_description_defaults_to_empty_string(self):
        equipment = Equipment()
        self.assertEquals(u"", equipment.description)
            
    def test_description_set_to_string(self):
        equipment = Equipment()
        try:
            equipment.description = "100$ Shoes"
        except(TypeError):
            pass
        else:
            self.fail("Should not be able to set description to non unicode string value.")
            
    def test_description_set_to_unicode_string(self):
        equipment = Equipment()
        equipment.description = u"Zapatos de €100"
        self.assertEquals(u"Zapatos de €100", equipment.description)
        
    def test_description_set_to_non_string(self):
        equipment = Equipment()
        try:
            equipment.description = 42
        except(TypeError):
            pass
        else:
            self.fail("Should not be able to set description to non string value.")
            
    def test_active_defaults_to_true(self):
        equipment = Equipment()
        self.assertTrue(equipment.active)
            
    def test_active_set_to_boolean(self):
        equipment = Equipment()
        equipment.active = False
        self.assertFalse(equipment.active)
            
    def test_active_set_to_non_boolean(self):
        equipment = Equipment()
        try:
            equipment.active = "test"
        except(TypeError):
            pass
        else:
            self.fail("Should not be able to set active to non-boolean value.")
    
    def test_life_expectancy_defaults_to_zero(self):
        equipment = Equipment()
        self.assertEqual(0, equipment.life_expectancy)
            
    def test_life_expectancy_set_to_integer(self):
        equipment = Equipment()
        equipment.life_expectancy = 2
        self.assertEquals(2, equipment.life_expectancy)
            
    def test_life_expectancy_set_to_numeric_string(self):
        equipment = Equipment()
        equipment.life_expectancy = "3"
        self.assertEquals(3, equipment.life_expectancy)

    def test_life_expectancy_set_to_non_numeric_string(self):
        equipment = Equipment()
        try:
            equipment.life_expectancy = "test"
        except(ValueError):
            pass
        else:
            self.fail("Should not be able to set life expectancy to non numeric value.")
    
    def test_prior_usage_defaults_to_zero(self):
        equipment = Equipment()
        self.assertEqual(0, equipment.prior_usage)
            
    def test_prior_usage_set_to_integer(self):
        equipment = Equipment()
        equipment.prior_usage = 2
        self.assertEquals(2, equipment.prior_usage)
            
    def test_prior_usage_set_to_numeric_string(self):
        equipment = Equipment()
        equipment.prior_usage = "3"
        self.assertEquals(3, equipment.prior_usage)

    def test_prior_usage_set_to_non_numeric_string(self):
        equipment = Equipment()
        try:
            equipment.prior_usage = "test"
        except(ValueError):
            pass
        else:
            self.fail("Should not be able to set life expectancy to non numeric value.")
            
    def test_notes_defaults_to_empty_string(self):
        equipment = Equipment()
        self.assertEquals(u"", equipment.notes)
            
    def test_notes_set_to_string(self):
        equipment = Equipment()
        try:
            equipment.notes = "100$ Shoes"
        except(TypeError):
            pass
        else:
            self.fail("Should not be able to set notes to non-unicode string value.")
            
    def test_notes_set_to_unicode_string(self):
        equipment = Equipment()
        equipment.notes = u"Zapatos de €100."
        self.assertEquals(u"Zapatos de €100.", equipment.notes)
        
    def test_notes_set_to_non_string(self):
        equipment = Equipment()
        try:
            equipment.notes = 42
        except(TypeError):
            pass
        else:
            self.fail("Should not be able to set notes to non-string value.")
            
    def test_equals_new_instances(self):
        equipment1 = Equipment()
        equipment2 = Equipment()
        self.assertNotEqual(equipment1, equipment2, "")
            
    def test_equals_instances_with_same_id(self):
        equipment1 = Equipment()
        equipment1.id = 1
        equipment2 = Equipment()
        equipment2.id = 1
        self.assertEqual(equipment1, equipment2, "Equipment instances with same id should be equal.")
            
    def test_equals_instances_with_different_ids(self):
        equipment1 = Equipment()
        equipment1.id = 1
        equipment2 = Equipment()
        equipment2.id = 2
        self.assertNotEqual(equipment1, equipment2, "Equipment instances with different ids should not be equal.")

class EquipmentServiceTest(unittest.TestCase):
    
    def setUp(self):
        profile = mock.Mock()
        profile.getValue = mock.Mock(return_value='memory')
        self.mock_ddbb = DDBB(profile)
        self.mock_ddbb.connect()
        self.mock_ddbb.create_tables()
        self.equipment_service = EquipmentService(self.mock_ddbb)
        
    def tearDown(self):
        self.mock_ddbb.disconnect()
    
    def test_get_equipment_item(self):
        self.mock_ddbb.insert("equipment", "life_expectancy,notes,description,prior_usage,active",
                              (500, u"Test notes.", u"Test Description", 200, 1))
        item = self.equipment_service.get_equipment_item(1)
        self.assertEquals(1, item.id)
        self.assertEquals("Test Description", item.description)
        self.assertTrue(item.active)
        self.assertEquals(500, item.life_expectancy)
        self.assertEquals(200, item.prior_usage)
        self.assertEquals("Test notes.", item.notes)
    
    def test_get_equipment_item_non_unicode(self):
        self.mock_ddbb.insert("equipment", "life_expectancy,notes,description,prior_usage,active",
                              (500, u"Test notes.", u"Test Description", 200, 1))
        item = self.equipment_service.get_equipment_item(1)
        self.assertEquals("Test Description", item.description)
        self.assertEquals("Test notes.", item.notes)
    
    def test_get_equipment_item_non_existant(self):
        item = self.equipment_service.get_equipment_item(1)
        self.assertEquals(None, item)
        
    def test_get_all_equipment(self):
        self.mock_ddbb.insert("equipment", "life_expectancy,notes,description,prior_usage,active",
                              (500, u"Test notes 1.", u"Test item 1", 200, 1))
        self.mock_ddbb.insert("equipment", "life_expectancy,notes,description,prior_usage,active",
                              (600, u"Test notes 2.", u"Test item 2", 300, 0))
        items = self.equipment_service.get_all_equipment()
        item = items[0]
        self.assertEquals(1, item.id)
        self.assertEquals("Test item 1", item.description)
        self.assertTrue(item.active)
        self.assertEquals(500, item.life_expectancy)
        self.assertEquals(200, item.prior_usage)
        self.assertEquals("Test notes 1.", item.notes)
        item = items[1]
        self.assertEquals(2, item.id)
        self.assertEquals("Test item 2", item.description)
        self.assertFalse(item.active)
        self.assertEquals(600, item.life_expectancy)
        self.assertEquals(300, item.prior_usage)
        self.assertEquals("Test notes 2.", item.notes)
        
    def test_get_all_equipment_non_existant(self):
        items = self.equipment_service.get_all_equipment()
        self.assertEquals([], items)
        
    def test_get_active_equipment(self):
        self.mock_ddbb.insert("equipment", "life_expectancy,notes,description,prior_usage,active",
                              (500, u"Test notes 1.", u"Test item 1", 200, 1))
        self.mock_ddbb.insert("equipment", "life_expectancy,notes,description,prior_usage,active",
                              (600, u"Test notes 2.", u"Test item 2", 300, 1))
        items = self.equipment_service.get_active_equipment()
        item = items[0]
        self.assertEquals(1, item.id)
        self.assertEquals("Test item 1", item.description)
        self.assertTrue(item.active)
        self.assertEquals(500, item.life_expectancy)
        self.assertEquals(200, item.prior_usage)
        self.assertEquals("Test notes 1.", item.notes)
        item = items[1]
        self.assertEquals(2, item.id)
        self.assertEquals("Test item 2", item.description)
        self.assertTrue(item.active)
        self.assertEquals(600, item.life_expectancy)
        self.assertEquals(300, item.prior_usage)
        self.assertEquals("Test notes 2.", item.notes)
        
    def test_get_active_equipment_non_existant(self):
        items = self.equipment_service.get_active_equipment()
        self.assertEquals([], items)
        
    def test_store_equipment(self):
        equipment = Equipment()
        equipment.description = u"test description"
        stored_equipment = self.equipment_service.store_equipment(equipment)
        self.assertEquals(1, stored_equipment.id)
        
    def test_store_equipment_duplicate_description(self):
        self.mock_ddbb.insert("equipment", "life_expectancy,notes,description,prior_usage,active",
                              (500, u"Test notes.", u"test item", 200, 1))
        equipment = Equipment()
        equipment.description = u"test item"
        try:
            self.equipment_service.store_equipment(equipment)
            self.fail("Should not be able to store new item with non-unique description.")
        except(EquipmentServiceException):
            pass
        
    def test_update_equipment(self):
        self.mock_ddbb.insert("equipment", "life_expectancy,notes,description,prior_usage,active",
                              (500, u"Test notes.", u"old description", 200, 1))
        equipment = self.equipment_service.get_equipment_item(1)
        equipment.description = u"new description"
        self.equipment_service.store_equipment(equipment)
        equipment = self.equipment_service.get_equipment_item(1)
        self.assertEquals("new description", equipment.description)
        
    def test_update_equipment_non_existant(self):
        equipment = Equipment()
        equipment.id = 1
        try:
            self.equipment_service.store_equipment(equipment)
            self.fail("Should not be able to update an item which did not previously exist.")
        except(EquipmentServiceException):
            pass
        
    def test_update_equipment_duplicate_description(self):
        self.mock_ddbb.insert("equipment", "life_expectancy,notes,description,prior_usage,active",
                              (500, u"Test notes.", u"test item", 200, 1))
        equipment = Equipment()
        equipment.id = 2
        equipment.description = u"test item"
        try:
            self.equipment_service.store_equipment(equipment)
            self.fail("Should not be able to change item description to non-unique value.")
        except(EquipmentServiceException):
            pass
        
    def test_get_equipment_usage(self):
        self.mock_ddbb.insert("equipment", "life_expectancy,notes,description,prior_usage,active",
                              (500, u"Test notes.", u"test item", 0, 1))
        self.mock_ddbb.insert("records", "distance", (250,))
        self.mock_ddbb.insert("record_equipment", "record_id,equipment_id", (1, 1))
        equipment = self.equipment_service.get_equipment_item(1)
        usage = self.equipment_service.get_equipment_usage(equipment)
        self.assertEquals(250, usage)
        
    def test_get_equipment_usage_none(self):
        self.mock_ddbb.insert("equipment", "life_expectancy,notes,description,prior_usage,active",
                              (500, u"Test notes.", u"test item", 0, 1))
        equipment = self.equipment_service.get_equipment_item(1)
        usage = self.equipment_service.get_equipment_usage(equipment)
        self.assertEquals(0, usage)

    def test_get_equipment_prior_usage(self):
        equipment = Equipment()
        equipment.id = 1
        equipment.prior_usage = 250
        usage = self.equipment_service.get_equipment_usage(equipment)
        self.assertEquals(250, usage)

    def test_get_equipment_prior_usage(self):
        equipment = Equipment()
        equipment.id = 1
        equipment.prior_usage = 250
        usage = self.equipment_service.get_equipment_usage(equipment)
        self.assertEquals(250, usage)

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



# -*- coding: utf-8 -*-

#Copyright (C) Nathan Jones ncjones@users.sourceforge.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from pytrainer.gui.color import ColorConverter
from pytrainer.util.color import Color
import gtk.gdk
import unittest

class ColorConverterTest(unittest.TestCase):
    
    def setUp(self):
        self._converter = ColorConverter()
    
    def test_convert_to_gdk_color_should_create_gdk_color_with_equivalent_rgb_values(self):
        color = Color(0xaaff33)
        gdk_color = self._converter.convert_to_gdk_color(color)
        self.assertEquals(0x3333, gdk_color.blue)
        self.assertEquals(0xffff, gdk_color.green)
        self.assertEquals(0xaaaa, gdk_color.red)
        
    def test_convert_to_color_should_create_color_with_equivalent_rgb_values(self):
        gdk_col = gtk.gdk.color_parse("#aaff33")
        color = self._converter.convert_to_color(gdk_col)
        self.assertEqual(0xaaff33, color.rgb_val)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
__author__ = "David García Granda - dgranda@gmail.com"
__copyright__ = "Copyright (C) 2013 David García Granda"
__license__ = "GPL v2 or later"

import unittest
import os
from lxml import etree
from imports.file_garminfit import garminfit

class GarminFitTest(unittest.TestCase):
    def test_parse_fit_file(self):
        try:
            current_path = os.path.dirname(os.path.abspath(__file__))
            data_path = os.path.dirname(os.path.dirname(os.path.dirname(current_path))) + "/"
            fit_file = current_path + "/sample.fit"
            garmin_fit = garminfit(None, data_path)
            xmldoc = etree.fromstring(garmin_fit.fromFIT2TCXv2(fit_file))
            valid_xml = garmin_fit.validate(xmldoc, "schemas/GarminTrainingCenterDatabase_v2.xsd")
            self.assertTrue(valid_xml)
        except():
            self.fail()

if __name__ == '__main__':
    unittest.main()



# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import logging
from gui.drawArea import DrawArea

import gtk

class RecordGraph:
    def __init__(self, vbox = None, window = None, combovalue = None, combovalue2 = None, btnShowLaps = None, tableConfig = None, pytrainer_main=None):
        logging.debug(">>")
        self.pytrainer_main = pytrainer_main
        self.drawarea = DrawArea(vbox, window)
        self.combovalue = combovalue
        self.combovalue2 = combovalue2
        self.showLaps = btnShowLaps
        self.config_table = tableConfig
        logging.debug("<<")

    def drawgraph(self,values,laps=None, y1limits=None, y1color=None, y1_linewidth=1):
        logging.debug(">>")
        #Get the config options
        for child in self.config_table.get_children():
            if child.get_name() == "spinbuttonY1Max":
                spinbuttonY1Max = child
            elif child.get_name() == "spinbuttonY1Min":
                spinbuttonY1Min = child
            elif child.get_name() == "colorbuttonY1LineColor":
                colorbuttonY1LineColor = child
            elif child.get_name() == "spinbuttonY1LineWeight":
                spinbuttonY1LineWeight = child

        xval = []
        yval = []
        xlab = []
        ylab = []
        tit = []
        col = []
        value_selected = self.combovalue.get_active()
        logging.debug("Value selected 1: "+ str(value_selected))
        value_selected2 = self.combovalue2.get_active()
        logging.debug("Value selected 2: "+ str(value_selected2))
        showLaps = self.showLaps.get_active()
        logging.debug("Show laps: "+ str(showLaps))
        #Determine left and right lap boundaries
        if laps is not None and showLaps:
            lapValues = []
            lastPoint = 0.0
            for lap in laps:
                thisPoint = float(lap['distance'])/1000.0 + lastPoint
                lapValues.append((lastPoint, thisPoint))
                lastPoint = thisPoint
        else:
            lapValues = None

        if value_selected < 0:
            self.combovalue.set_active(0)
            value_selected = 0

        if value_selected2 < 0:
            self.combovalue2.set_active(0)
            value_selected2 = 0
        xvalues, yvalues = self.get_values(values,value_selected)
        max_yvalue = max(yvalues)
        min_yvalue = min(yvalues)
        xlabel,ylabel,title,color = self.get_value_params(value_selected)
        if y1color is not None:
            _color = gtk.gdk.Color(y1color)
            color = y1color
        else:
            _color = gtk.gdk.Color(color)

        xval.append(xvalues)
        yval.append(yvalues)
        if value_selected2 > 0:
            xlab.append("")
        else:
            xlab.append(xlabel)
        ylab.append(ylabel)
        tit.append(title)
        col.append(color)

        #_color = gtk.gdk.Color(color)
        colorbuttonY1LineColor.set_color(_color)

        if value_selected2 > 0:
            value_selected2 = value_selected2-1
            xlabel,ylabel,title,color = self.get_value_params(value_selected2)
            xvalues,yvalues = self.get_values(values,value_selected2)
            max_yvalue=max(max(yvalues), max_yvalue)
            min_yvalue=min(min(yvalues), min_yvalue)
            xval.append(xvalues)
            yval.append(yvalues)
            xlab.append(xlabel)
            ylab.append(ylabel)
            tit.append("")
            col.append(color)
        logging.info("To show: tit: "+str(tit)+" | col: "+str(col)+" | xlab: "+str(xlab)+" | ylab: "+str(ylab))
        #self.drawPlot(xvalues,yvalues,xlabel,ylabel,title,color,zones)
        plot_stats = self.drawarea.drawPlot(xval,yval,xlab,ylab,tit,col,None,lapValues, ylimits=y1limits, y1_linewidth=y1_linewidth)
        ymin = plot_stats['y1_min']
        ymax = plot_stats['y1_max']
        y1_linewidth = plot_stats['y1_linewidth']

        max_yvalue = max(max_yvalue, ymax)
        min_yvalue = min(min_yvalue, ymin)
        adjY1Min = gtk.Adjustment(value=ymin, lower=min_yvalue,upper=max_yvalue, step_incr=1, page_incr=10)
        adjY1Max = gtk.Adjustment(value=ymax, lower=min_yvalue,upper=max_yvalue, step_incr=1, page_incr=10)
        spinbuttonY1Min.set_adjustment(adjY1Min)
        spinbuttonY1Max.set_adjustment(adjY1Max)
        spinbuttonY1Min.set_value(ymin)
        spinbuttonY1Max.set_value(ymax)
        spinbuttonY1LineWeight.set_value(y1_linewidth)

        logging.debug("<<")

    def get_value_params(self,value):
        if value == 0:
            return _("Distance (km)"),_("Height (m)"),_("Stage Profile"),"#000000"
        if value == 1:
            return _("Distance (km)"),_("Speed (Km/h)"),_("Speed"),"#00ff00"
        if value == 2:
            return _("Distance (km)"),_("Pace (min/km)"),_("Pace"),"#0000ff"
        if value == 3:
            return _("Distance (km)"),_("Beats (bpm)"),_("Heart Rate"),"#ff0000"
        if value == 4:
            return _("Distance (km)"),_("Cadence (rpm)"),_("Cadence"),"#7B3F00"
        if value == 5:
            return _("Distance (km)"),_("Beats (%)"),_("Beats"),"#ff0000"
        if value == 6:
            return _("Distance (km)"),_("Zone"),_("Zone"),"#ff0000"


    def get_values(self,values, value_selected):
        logging.debug(">>")
        xvalue = []
        yvalue = []
        zones = self.pytrainer_main.profile.getZones()
        for value in values:
            xvalue.append(value[0])
            if value_selected==0:
                yvalue.append(value[1])
            if value_selected==1:
                yvalue.append(value[3])
            if value_selected==2:
                try:
                    yvalue.append(60/value[3])
                except:
                    yvalue.append(0)
            if value_selected==3:
                yvalue.append(value[6])
            if value_selected==4:
                yvalue.append(value[7])
            if value_selected==5:
                if value[6] <= zones[4][0]:
                    yvalue.append(50.0*value[6]/zones[4][0])
                else:
                    yvalue.append(50.0+50.0*((value[6]-zones[4][0])/(zones[0][1]-zones[4][0])))
            if value_selected==6:
                if value[6] <= zones[4][0]:
                    yvalue.append(1.0*value[6]/zones[4][0])
                else:
                    yvalue.append(1.0+5.0*((value[6]-zones[4][0])/(zones[0][1]-zones[4][0])))
        logging.debug("<<")
        return xvalue,yvalue

    def getFloatValue(self, value):
        try:
            return float(value)
        except:
            return float(0)




# -*- coding: utf-8 -*-

#Copyright (C) Nathan Jones ncjones@users.sourceforge.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from pytrainer.util.color import Color, color_from_hex_string
import unittest

class ColorTest(unittest.TestCase):
    
    def test_constructor_should_accept_integer(self):
        color = Color(12345)
        self.assertEqual(12345, color.rgb_val)
    
    def test_constructor_should_accept_integer_string(self):
        color = Color("12345")
        self.assertEqual(12345, color.rgb_val)
    
    def test_constructor_should_not_accept_non_integer_string(self):
        try:
            Color("ff00ff")
        except(ValueError):
            pass
        else:
            self.fail()
    
    def test_constructor_should_not_accept_none(self):
        try:
            Color(None)
        except(TypeError):
            pass
        else:
            self.fail()
    
    def test_constructor_should_not_accept_negative_value(self):
        try:
            Color(-1)
        except(ValueError):
            pass
        else:
            self.fail()
            
    def test_constructor_should_not_accept_value_over_24_bit(self):
        try:
            Color(2 ** 24)
        except(ValueError):
            pass
        else:
            self.fail()

    def test_rgb_value_should_default_to_0(self):
        color = Color()
        self.assertEqual(0, color.rgb_val)
            
    def test_rgb_value_should_be_read_only(self):
        color = Color()
        try:
            color.rgb_val = 1
        except(AttributeError):
            pass
        else:
            self.fail()
            
    def test_rgba_value_should_be_rgb_value_with_two_trailing_zero_hex_digits(self):
        color = Color(0x1177ff)
        self.assertEquals(0x1177ff00, color.rgba_val)
        
    def test_to_hex_string_should_create_six_digit_hex_value(self):
        color = Color(0xfab)
        self.assertEquals("000fab", color.to_hex_string())
        
    def test_color_from_hex_string_should_correctly_decode_hex_value(self):
        color = color_from_hex_string("fab")
        self.assertEquals(0xfab, color.rgb_val)



# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from pytrainer.gui.dialogs import warning_dialog
import gtk
import logging

class Warning(object):
    def __init__(self, data_path = None, okmethod = None, okparams = None, cancelmethod = None, cancelparams = None):
        logging.warning("Deprecated Warning class called")
        self.okmethod = okmethod
        self.cancelmethod = cancelmethod
        self.okparams = okparams
        self.cancelparams = cancelparams
        self.text = ""
        self.title = "Warning"

    def set_title(self, title):
        self.title = title

    def set_text(self, msg):
        self.text = msg

    def on_accept_clicked(self):
        if self.okparams != None:
            num = len(self.okparams)
            if num==0:
                self.okmethod()
            if num==1:
                self.okmethod(self.okparams[0])
            if num==2:
                self.okmethod(self.okparams[0],self.okparams[1])

    def on_cancel_clicked(self):
        if self.cancelparams != None:
            num = len(self.cancelparams)
            if num==0:
                self.cancelmethod()
            if num==1:
                self.cancelmethod(self.cancelparams[0])
            if num==2:
                self.cancelmethod(self.cancelparams[0], self.cancelparams[1])

    def run(self):
        if self.okmethod:
            response = warning_dialog(text=self.text, title=self.title, cancel=True)
        else:
            response = warning_dialog(text=self.text, title=self.title, cancel=False)
        if response == gtk.RESPONSE_OK:
            self.on_accept_clicked()
        elif response == gtk.RESPONSE_CANCEL:
            self.on_cancel_clicked()



# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import logging
import gtk

class GraphData:
    '''
    Class to hold data and formating for graphing via matplotlib
    '''
    def __init__(self, title=None, ylabel=None, xlabel=None):
        logging.debug('>>')
        self.title = title
        self.ylabel = ylabel
        self.xlabel = xlabel
        self.labels = []
        self.colors = []
        self.x_values = []
        self.bar_bottoms = []
        self.bar_widths = []
        self.y_values = []
        self.linewidth = 1
        self.linecolor = '#ff0000'
        self.y2linecolor = '#ff0000'
        self.max_x_value = None
        self.min_x_value = None
        self.max_y_value = None
        self.min_y_value = None
        self.graphType = "plot"
        self.show_on_y1 = False
        self.show_on_y2 = False
        logging.debug('<<')
            
    def addBars(self, x=None, y=None):
        if x is None or y is None:
            #logging.debug("Must supply both x and y data points, got x:'%s' y:'%s'" % (str(x), str(y)))
            return
        #print('Adding point: %s %s' % (str(x), str(y)))
        if len(self.x_values) == 0:
            #First bar, so start a 0
            self.x_values.append(0)
        else:
            #Second or subsequent bar, so start at last point
            #Which is previous left+width
            items = len(self.x_values)
            last_left = self.x_values[items-1]
            last_width = self.bar_widths[items-1]
            new_left = last_left+last_width
            self.x_values.append(new_left)
        self.bar_widths.append(x)            
        self.y_values.append(y)
        self.bar_bottoms.append(0)
        
    def addPoints(self, x=None, y=None, label=None, color=None):
        #if x is None or y is None or x is "":
        if not x or not y:
            #logging.debug("Must supply both x and y data points, got x:'%s' y:'%s'" % (str(x), str(y)))
            return
        #print('Adding point: %s %s' % (str(x), str(y)))
        self.x_values.append(x)
        self.y_values.append(y)
        if label is not None:
            self.labels.append(label)
        if color is not None:
            self.colors.append(color)
        if self.max_x_value is None or x > self.max_x_value:
            self.max_x_value = x
        if self.min_x_value is None or x < self.min_x_value:
            self.min_x_value = x
        if self.max_y_value is None or y > self.max_y_value:
            self.max_y_value = y
        if self.min_y_value is None or y < self.min_y_value:
            self.min_y_value = y

    def get_color(self, color):
        '''
        
        '''
        if color is None:
            return None
        try:
            #Generate 13 digit color string from supplied color
            col = gtk.gdk.color_parse(color).to_string()
        except ValueError:
            logging.debug("Unable to parse color from '%s'" % color)
            return None
                
        #Create matplotlib color string
        _color = "#%s%s%s" % (col[1:3], col[5:7], col[9:11])
        #logging.debug("%s color saved as: %s" % (color, _color))
        return _color
        
    def set_color(self, y1color, y2color = None):
        ''' 
            Helper function to set the line color
            need as some gtk.gdk color can be invalid for matplotlib
        '''
        _color = self.get_color(y1color)
        _color2 = self.get_color(y2color)
        #if _color is not None:
        self.linecolor = _color
        #if _color2 is not None:
        self.y2linecolor = _color2

    def __len__(self):
        if self.x_values is None:
            return None
        return len(self.x_values)
        
    def __str__(self):
        return '''
Title: %s
ylabel: %s
xlabel: %s
linewidth: %d
linecolor: %s
graphType: %s
show on y1: %s
show on y2: %s
x min max: %s %s
y min max: %s %s
x values: %s
y values: %s''' % (self.title,
    self.ylabel,
    self.xlabel,
    self.linewidth,
    self.linecolor,
    self.graphType,
    str(self.show_on_y1),
    str(self.show_on_y2),
    str(self.min_x_value), str(self.max_x_value),
    str(self.min_y_value), str(self.max_y_value),
    str(self.x_values),
    str(self.y_values)
    )

    






#!/usr/bin/env python

import gtk
import subprocess
import re

class gpx2garmin:
    def __init__(self, parent = None, pytrainer_main = None, conf_dir = None, options = None):
        self.limit = options["gpx2garminmaxpoints"]
        self.device = options["gpx2garmindevice"]
        self.conf_dir = conf_dir
        self.gpxfile = None
        self.tmpgpxfile = "/tmp/_gpx2garmin.gpx"
        self.pytrainer_main = pytrainer_main

    def prepareGPX(self):
        ' create an output file and insert a name into the <trk> stanza '
        f = open(self.tmpgpxfile, 'w')
        name = 'default'
        for line in open(self.gpxfile):
            if re.search('^<name>.+</name>$', line):
                name = line
            elif re.search('^<trk>$', line):
                line = '%s%s' % (line, name)
            f.write(line)
        f.close()
        
    def exportGPX(self):
        ' export the GPX file using gpsbabel '
        cmd = ["gpsbabel", "-t", "-i", "gpx", "-f", self.tmpgpxfile, "-o", "garmin"]
        if self.limit is not None:
            cmd = cmd + ["-x", "simplify,count=%s" % self.limit]
        cmd = cmd + ["-F", self.device]
        return subprocess.call(cmd)

    def run(self, id, activity=None):
        ' main extension method '
        self.gpxfile = "%s/gpx/%s.gpx" % (self.conf_dir, id)
        self.log = "Export "
        try:
            self.prepareGPX()
            if self.exportGPX() == 1:
                # assume no device present?
                self.log = self.log + "failed - no device present?"
            else:
                self.log = self.log + "succeeded!" 
        except:
            self.log = self.log + "failed!"
        md = gtk.MessageDialog(self.pytrainer_main.windowmain.window1, gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, self.log)
        md.set_title(_("gpx2garmin Extension"))
        md.set_modal(False)
        md.run()
        md.destroy()



# -*- coding: iso-8859-1 -*-

#Copyright (C) Nathan Jones ncjones@users.sourceforge.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import unittest
import sqlalchemy
import pytrainer.upgrade.versions.version014 as version014

class UpgradeTest(unittest.TestCase):
    
    def setUp(self):
        self._engine = sqlalchemy.create_engine("sqlite:///:memory:")
        self._metadata = sqlalchemy.MetaData()
        self._metadata.bind = self._engine
        self._sports_table = sqlalchemy.Table("sports", self._metadata,
            sqlalchemy.Column("id_sports", sqlalchemy.Integer, primary_key=True),
            sqlalchemy.Column("name", sqlalchemy.String(100)),
            sqlalchemy.Column("weight", sqlalchemy.Float),
            sqlalchemy.Column("color", sqlalchemy.String(6)),
            sqlalchemy.Column("met", sqlalchemy.Float),
            sqlalchemy.Column("max_pace", sqlalchemy.Integer)
        )
        self._metadata.drop_all()
        self._metadata.create_all()
        
    def upgradeAndAssert(self, original, expected):
        self._engine.execute(sqlalchemy.text("insert into sports (id_sports, name, weight, color, met, max_pace)"
                                             " values (:id, :name, :weight, :color, :met, :max_pace)"),
                                           id= 1,
                                           name= "Test Sport",
                                           weight= original["weight"],
                                           color= original["color"],
                                           met= original["met"],
                                           max_pace= original["max_pace"])
        version014.upgrade(self._engine)
        result = self._engine.execute(self._sports_table.select(self._sports_table.c.id_sports==1))
        (_, _, weight, color, met, max_pace) = result.fetchone()
        result.close()
        self.assertEqual(expected["weight"], weight)
        self.assertEqual(expected["color"], color)
        self.assertEqual(expected["met"], met)
        self.assertEqual(expected["max_pace"], max_pace)
        
    def testUpgradeFromNullValues(self):
        original = { "weight": None, "color": None, "met": None, "max_pace": None }
        expected = { "weight": 0.0, "color": "0000ff", "met": None, "max_pace": None }
        self.upgradeAndAssert(original, expected)
        
    def testUpgradeFromZeroValues(self):
        original = { "weight": 0, "color": 0, "met": 0, "max_pace": 0 }
        expected = { "weight": 0.0, "color": "0", "met": 0, "max_pace": None }
        self.upgradeAndAssert(original, expected)
        
    def testUpgradeFromEmptyValues(self):
        original = { "weight": "", "color": "", "met": "", "max_pace": "" }
        expected = { "weight": 0.0, "color": "0000ff", "met": None, "max_pace": None }
        self.upgradeAndAssert(original, expected)
        
    def testUpgradeFromNegativeValues(self):
        original = { "weight": -1, "color": -1, "met": -1, "max_pace": -1 } 
        expected = { "weight": 0.0, "color": "-1", "met": None, "max_pace": None }
        self.upgradeAndAssert(original, expected)
        
    def testUpgradeFromValidValues(self):
        original = { "weight": 3.4, "color": "abc123", "met": 45.6, "max_pace": 123 }
        expected = { "weight": 3.4, "color": "abc123", "met": 45.6, "max_pace": 123 }
        self.upgradeAndAssert(original, expected)



# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net
# Jakinbidea & Grupo Ikusnet Developer
# vud1@grupoikusnet.com

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from SimpleGladeApp import SimpleBuilderApp
import gtk
import gobject
import logging

class DialogSelectTrack(SimpleBuilderApp):
	def __init__(self, data_path = None, tracks = None, okmethod = None, gpx = None):
		logging.debug(">>")
		self.okmethod = okmethod
		self.tracks = tracks
		self.gpx = gpx
		SimpleBuilderApp.__init__(self, "selecttrackdialog.ui")
		logging.debug("<<")

	def new(self):		
		#preparamos la lista con los tracks disponibles:
		logging.debug(">>")
		column_names=[_("Track Name"), _("Date")]
		self.create_treeview(self.trkpTreeView,column_names)
		self.actualize_treeview(self.trkpTreeView,self.tracks)
		logging.debug("<<")

	def on_ok_clicked(self,widget):
		logging.debug(">>")
		selected,iter = self.trkpTreeView.get_selection().get_selected()
		if iter:
			trackname = selected.get_value(iter,0)
			logging.debug("selected track: "+trackname)
		self.okmethod(self.gpx,trackname)
		self.closewindow()
		logging.debug("<<")
	
	def on_cancel_clicked(self,widget):
		logging.debug("--")
		self.closewindow()

	def closewindow(self):
		logging.debug("--")
		self.selecttrackdialog.hide()
		#self.selecttrackdialog = None
		self.quit()	
	
	def create_treeview(self,treeview,column_names):
		logging.debug(">>")
		i=0
		for column_index, column_name in enumerate(column_names):
			column = gtk.TreeViewColumn(column_name, gtk.CellRendererText(), text=column_index)
			column.set_resizable(True)
			column.set_sort_column_id(i)
			treeview.append_column(column)
			i+=1
		logging.debug("<<")
	
	def actualize_treeview(self, treeview, record_list):
		logging.debug(">>")
		iterOne = False
		store = gtk.ListStore(
			gobject.TYPE_STRING,
			gobject.TYPE_STRING,
			object)
		for i in record_list:
			iter = store.append()
			if not iterOne:
				iterOne = iter
			store.set (
				iter,
				0, str(i[0]),
				1, str(i[1]))
		treeview.set_model(store)
		if iterOne:
			treeview.get_selection().select_iter(iterOne)
		logging.debug("<<")



#Copyright (C) Arto Jantunen <viiru@iki.fi>

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import unittest
import datetime
from dateutil.tz import tzutc, tzlocal
from pytrainer.lib.date import second2time, time2second, time2string, unixtime2date, getNameMonth, getDateTime

class DateFunctionTest(unittest.TestCase):

    def test_second2time(self):
        tmp = second2time(3912)
        self.assertEqual((1, 5, 12), tmp)

    def test_time2second(self):
        tmp = time2second((1, 5, 12))
        self.assertEqual(3912, tmp)

    def test_time2string(self):
        tmp = time2string((2015, 11, 24))
        self.assertEqual('2015-11-24', tmp)

    def test_getNameMonth(self):
        monthname, daysinmonth = getNameMonth(datetime.date(2015, 11, 24))
        self.assertEqual('November', monthname)
        self.assertEqual(30, daysinmonth)

    def test_unixtime2date(self):
        tmp = unixtime2date(1448378940)
        self.assertEqual('2015-11-24', tmp)

    def test_getDateTime(self):
        utctime, localtime = getDateTime('Tue Nov 24 17:29:05 UTC 2015')
        self.assertEqual(datetime.datetime(2015, 11, 24, 17, 29, 5, tzinfo=tzutc()), utctime)
        self.assertEqual(datetime.datetime(2015, 11, 24, 19, 29, 5, tzinfo=tzlocal()), localtime)



from pytrainer.upgrade.context import UPGRADE_CONTEXT
import pytrainer.upgrade.versions.version014 as version14
import sqlalchemy

def upgrade(migrate_engine=None):
    if migrate_engine is None:
        # sqlalchemy-migrate 0.5.4 does not provide migrate engine to upgrade scripts
        migrate_engine = sqlalchemy.create_engine(UPGRADE_CONTEXT.db_url)
    version14.upgrade(migrate_engine)


# -*- coding: iso-8859-1 -*-

#Copyright (C) Sigurður H. Pálsson shp@itn.is

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from timegraph import TimeGraph

class TotalGraph(TimeGraph):

    value_params = [
        (_("year"),_("Distance (km)"),_("Annual Distance"),"y"),
        (_("year"),_("Time (hours)"), _("Annual Time"),"b"),
        (_("year"),_("Average Heart Rate (bpm)"), _("Annual Average Heart Rate"),"r"),
        (_("year"),_("Average Speed (km/h)"), _("Annual Average Speed"),"g"),
        (_("year"),_("Calories"), _("Annual Calories"),"b"),
    ]

    def __init__(self, sports, vbox = None, window = None, combovalue = None, combovalue2 = None, main = None):
        TimeGraph.__init__(self, sports, vbox=vbox, window=window, main=main)
        self.combovalue = combovalue
        self.combovalue2 = combovalue2
        self.KEY_FORMAT = "%Y"
        self.SPORT_FIELD = 4

    def getYears(self, yvalues):
        years = set()
        for s in yvalues.values():
            years |= set([str(x) for x in xrange(int(min(s.keys())), int(max(s.keys()))+1)])
        return sorted(list(years))

    def drawgraph(self,values):
        TimeGraph.drawgraph(self, values, x_func=self.getYears)

    def getValue(self,record,value_selected):
        #hacemos una relacion entre el value_selected y los values / we make a relation between value_selected and the values
        conv = {
            0: 1, #value 0 es kilometros (1)
            1: 6, #value 1 es tiempo (2)
            2: 7, #value 2 es pulsaciones(3)
            3: 2, #value 3 es media (2)
            4: 8 #value 4 es calorias(6)
            }
        value_sel = conv[value_selected]
        #si la opcion es tiempo lo pasamos a horas / if the option is time we passed it to hours
        if (value_sel == 6):
            return self.getFloatValue(record[value_sel])/3600
        else:
            return self.getFloatValue(record[value_sel])
    




# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net
# Modified by dgranda

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# Added to support python installations older than 2.6
from __future__ import with_statement

import logging
import os
import StringIO
from lxml import etree

from pytrainer.lib.xmlUtils import XMLParser
from pytrainer.gui.dialogs import fileChooserDialog, guiFlush

class garmintools():
	def __init__(self, parent = None, validate=False):
		self.parent = parent
		self.pytrainer_main = parent.pytrainer_main
		self.tmpdir = self.pytrainer_main.profile.tmpdir
		self.data_path = os.path.dirname(__file__)
		self.validate = validate
		self.sport = self.getConfValue("Force_sport_to")

	def getConfValue(self, confVar):
		info = XMLParser(self.data_path+"/conf.xml")
		code = info.getValue("pytrainer-plugin","plugincode")
		plugindir = self.pytrainer_main.profile.plugindir
		if not os.path.isfile(plugindir+"/"+code+"/conf.xml"):
			value = None
		else:
			info = XMLParser(plugindir+"/"+code+"/conf.xml")
			value = info.getValue("pytrainer-plugin",confVar)
		return value

	def run(self):
		logging.debug(">>")
		# able to select multiple files....
		selectedFiles = fileChooserDialog(title="Choose a garmintools dump file (or files) to import", multiple=True).getFiles()
		guiFlush()
		importfiles = []
		if not selectedFiles: #Nothing selected
			return importfiles
		for filename in selectedFiles:
			if self.valid_input_file(filename):
				#Garmin dump files are not valid xml - need to load into a xmltree
				#read file into string
				with open(filename, 'r') as f:
					xmlString = f.read()
				fileString = StringIO.StringIO("<root>"+xmlString+"</root>")
				#parse string as xml
				tree = etree.parse(fileString)
				if not self.inDatabase(tree):
					sport = self.getSport(tree)
					gpxfile = "%s/garmintools-%d.gpx" % (self.tmpdir, len(importfiles))
					self.createGPXfile(gpxfile, tree)
					importfiles.append((gpxfile, sport))
				else:
					logging.debug("%s already in database. Skipping import." % (filename,) )
			else:
				logging.info("File %s failed validation" % (filename))
		logging.debug("<<")
		return importfiles

	def valid_input_file(self, filename):
		""" Function to validate input file if requested"""
		if not self.validate:  #not asked to validate
			logging.debug("Not validating %s" % (filename) )
			return True
		else:
			print "Cannot validate garmintools dump files yet"
			logging.debug("Cannot validate garmintools dump files yet")
			return True
			'''xslfile = os.path.realpath(self.parent.parent.data_path)+ "/schemas/GarminTrainingCenterDatabase_v2.xsd"
			from lib.xmlValidation import xmlValidator
			validator = xmlValidator()
			return validator.validateXSL(filename, xslfile)'''

	def inDatabase(self, tree):
		#comparing date and start time (sport may have been changed in DB after import)
		time = self.detailsFromFile(tree)
		if self.pytrainer_main.ddbb.select("records","*","date_time_utc=\"%s\"" % (time)):
			return True
		else:
			return False

	def getSport(self, tree):
		#return sport from file or overide if present
		if self.sport:
			return self.sport
		root = tree.getroot()
		sportElement = root.find(".//run")
		try:
			sport = sportElement.get("sport")
			sport = sport.capitalize()
		except:
			sport = "import"
		return sport

	def detailsFromFile(self, tree):
		root = tree.getroot()
		#Find first point
		pointElement = root.find(".//point")
		if pointElement is not None:
			#Try to get time from point
			time = pointElement.get("time")
			print "#TODO first time is different from time used by gpsbabel and has locale embedded: " + time
			return time
		return None

	def createGPXfile(self, gpxfile, tree):
		""" Function to transform a Garmintools dump file to a valid GPX+ file
		"""
		xslt_doc = etree.parse(self.data_path+"/translate.xsl")
		transform = etree.XSLT(xslt_doc)
		result_tree = transform(tree)
		result_tree.write(gpxfile, xml_declaration=True, encoding='UTF-8')




# -*- coding: utf-8 -*-

#Copyright (C) David García Granda dgranda@users.sourceforge.net
#Copyright (C) Kevin Dwyer kevin@pheared.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# Need to set this stuff up so that the translation functions work.  Seems like
# the module that needs translation ought to have some way of setting this up.
import pytrainer.lib.localization
pytrainer.lib.localization.initialize_gettext("../../locale")

import unittest
import os
from lxml import etree
from pytrainer.lib.gpx import Gpx

class GpxTest(unittest.TestCase):

    def setUp(self):
        self.tmp_files = []
        
    def tearDown(self):
        for file_name in self.tmp_files:
            try:
                os.remove(file_name)
            except:
                pass

    def test_get_laps_old(self):
        orig_laps = [
            ("1264.66","42.84154594","-2.68554166","426","5000.71875","42.83547375","-2.68631422","active","170","177","4.93775940","manual"),
            ("1279.71","42.86093295","-2.66849270","445","5162.37109","42.84155038","-2.68552473","active","176","179","5.10653210","manual"),
            ("1263.54","42.83505499","-2.67709371","423","4882.18457","42.86094376","-2.66848792","active","176","179","4.37805939","manual"),
            ("1525.68","42.84018606","-2.68670272","426","4973.64746","42.83504661","-2.67710888","active","167","181","4.52464294","manual"),
            ("374.23","42.83771038","-2.68647373","96","1098.94531","42.84018849","-2.68670733","active","159","163","4.30066299","manual")]
        try:
            xml_file = os.path.dirname(os.path.abspath(__file__)) + "/gpxplus_sample_old.gpx"
            gpx = Gpx(None, None) # avoid launching _getValues
            gpx.tree = etree.ElementTree(file = xml_file).getroot()
            gpx_laps = gpx.getLaps()
            self.assertEquals(orig_laps, gpx_laps)
        except():
            self.fail()

    def test_get_laps(self):
        orig_laps = [
            ("311.31","43.53781521","-5.63955233","81","1000.000000","43.54065232","-5.65094300","active","158","178","3.52099586","distance"),
            ("337.85","43.53220135","-5.63737772","83","1000.000000","43.53780859","-5.63955157","active","149","153","4.07694530","distance"),
            ("342.13","43.52516323","-5.64443462","87","1000.000000","43.53218752","-5.63737328","active","150","154","3.13006544","distance"),
            ("353.81","43.52035671","-5.65329663","86","1000.000000","43.52515301","-5.64443881","active","146","150","3.00786400","distance"),
            ("352.61","43.51314962","-5.65532908","87","1000.000000","43.52035412","-5.65329814","active","148","158","3.17764997","distance"),
            ("354.17","43.52061689","-5.65409191","87","1000.000000","43.51314115","-5.65533193","active","142","149","3.52461219","distance"),
            ("343.65","43.52592498","-5.64510651","86","1000.000000","43.52062519","-5.65408990","active","144","147","3.04636431","distance"),
            ("366.95","43.53079587","-5.63821390","83","1000.000000","43.52592733","-5.64509553","active","142","150","3.21967506","distance"),
            ("345.69","43.53726536","-5.63784711","87","1000.000000","43.53081406","-5.63820661","active","146","150","4.38874722","distance"),
            ("330.64","43.54042768","-5.64873822","86","1000.000000","43.53726494","-5.63783269","active","149","154","3.56236672","distance"),
            ("41.96","43.54054570","-5.65028653","11","132.227539","43.54043892","-5.64874199","active","150","152","3.40324497","manual")]
        try:
            xml_file = os.path.dirname(os.path.abspath(__file__)) + "/gpxplus_sample.gpx"
            gpx = Gpx(None, None) # avoid launching _getValues
            gpx.tree = etree.ElementTree(file = xml_file).getroot()
            gpx_laps = gpx.getLaps()
            self.assertEquals(orig_laps, gpx_laps)
        except():
            self.fail()

    def test_missing_tracks(self):
        trkdata = """<?xml version="1.0" encoding="UTF-8" standalone="no" ?>
<gpx xmlns="http://www.topografix.com/GPX/1/1" creator="" version="1.1" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.topografix.com/GPX/1/1 http://www.topografix.com/GPX/1/1/gpx.xsd">
</gpx>
"""
        
        # Write a GPX file with no tracks
        file_name = "test-missing.gpx"
        tmpf = file(file_name,'w')
        tmpf.write(trkdata)
        tmpf.close()
        self.tmp_files.append(file_name)
        
        try:
            g = Gpx(filename=file_name)
        except IndexError:
            self.fail("Gpx parser crashed on file without tracks")

    def test_missing_name(self):
        trkdata = """<?xml version="1.0" encoding="UTF-8" standalone="no" ?>
<gpx xmlns="http://www.topografix.com/GPX/1/1" creator="" version="1.1" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.topografix.com/GPX/1/1 http://www.topografix.com/GPX/1/1/gpx.xsd">
<trk></trk>
</gpx>
"""
        
        # Write a GPX file with a nameless track
        file_name = "test-noname.gpx"
        tmpf = file(file_name,'w')
        tmpf.write(trkdata)
        tmpf.close()
        self.tmp_files.append(file_name)
        
        try:
            g = Gpx(filename=file_name)
        except IndexError:
            self.fail("Gpx parser crashed on file with a nameless track")

if __name__ == '__main__':
    unittest.main()
